@@ -274,8 +274,8 @@ public class CastToList extends UnaryExpression {
      */
 
     @Override
-    public int hashCode() {
-        return super.hashCode() ^ targetType.hashCode();
+    public int computeHashCode() {
+        return super.computeHashCode() ^ targetType.hashCode();
     }
 
     /**
