@@ -31,6 +31,7 @@ public class ErrorExpression extends Expression {
     private String message;
     private String errorCode;
     private boolean isTypeError;
+    private Expression original;
 
     /**
      * This constructor is never executed, but it is used in the expression parser
@@ -89,6 +90,15 @@ public class ErrorExpression extends Expression {
         return exception == null ? errorCode : exception.getErrorCodeLocalPart();
     }
 
+    /**
+     * Set the original expression (for diagnostics)
+     * @param original the expression that this error expression replaces
+     */
+
+    public void setOriginalExpression(Expression original) {
+        this.original = original;
+    }
+
     /**
      * Type-check the expression.
      */
@@ -125,10 +135,14 @@ public class ErrorExpression extends Expression {
         if (exception != null) {
             // copy the exception for thread-safety, because we want to add context information
             XPathException err = new XPathException(exception.getMessage());
-            err.setLocation(getLocation());
+            err.setLocation(exception.getLocator());
+            err.maybeSetLocation(getLocation());
             err.setErrorCodeQName(exception.getErrorCodeQName());
-            err.setXPathContext(context);
+            err.setXPathContext(exception.getXPathContext());
+            err.maybeSetContext(context);
             err.setIsTypeError(exception.isTypeError());
+            err.setIsStaticError(exception.isStaticError());
+            err.setIsGlobalError(exception.isGlobalError());
             throw err;
         } else {
             XPathException err = new XPathException(message);
@@ -183,10 +197,12 @@ public class ErrorExpression extends Expression {
     public Expression copy(RebindingMap rebindings) {
         if (exception != null) {
             ErrorExpression e2 = new ErrorExpression(exception);
+            e2.setOriginalExpression(original);
             ExpressionTool.copyLocationInfo(this, e2);
             return e2;
         } else {
             ErrorExpression e2 = new ErrorExpression(message, errorCode, isTypeError);
+            e2.setOriginalExpression(original);
             ExpressionTool.copyLocationInfo(this, e2);
             return e2;
         }
@@ -206,12 +222,20 @@ public class ErrorExpression extends Expression {
 
     @Override
     public String toString() {
-        return "error(\"" + message + "\")";
+        if (original != null) {
+            return original.toString();
+        } else {
+            return "error(\"" + message + "\")";
+        }
     }
 
     @Override
     public String toShortString() {
-        return toString();
+        if (original != null) {
+            return original.toShortString();
+        } else {
+            return "error(\"" + message + "\")";
+        }
     }
 
     /**
