@@ -56,7 +56,7 @@ import java.util.*;
  * no subexpressions.</p>
  */
 
-public abstract class Expression implements IdentityComparable {
+public abstract class Expression implements IdentityComparable, ExportAgent {
 
     public static final int EVALUATE_METHOD = 1;
     public static final int ITERATE_METHOD = 2;
@@ -75,6 +75,7 @@ public abstract class Expression implements IdentityComparable {
     private int evaluationMethod;
     private Map<String, Object> extraProperties;
     private int cost = -1;
+    private int cachedHashCode = -1;
 
 //    public int serial;  // used to identify expressions for internal diagnostics
 //    private static int nextSerial = 0;
@@ -230,6 +231,10 @@ public abstract class Expression implements IdentityComparable {
                 throw new IllegalStateException("Incorrect parent pointer in " + parent.toShortString() +
                                                         " subexpression " + o.getChildExpression().toShortString());
             }
+            if (o.getChildExpression().getRetainedStaticContext() == null) {
+                throw new IllegalStateException("No retained static context in " + parent.toShortString() +
+                                                        " subexpression " + o.getChildExpression().toShortString());
+            }
             o.getChildExpression().verifyParentPointers();
         }
         return this;
@@ -299,8 +304,11 @@ public abstract class Expression implements IdentityComparable {
 
     public boolean isLiftable(boolean forStreaming) {
         int p = getSpecialProperties();
+        int d = getDependencies();
         return (p & StaticProperty.NON_CREATIVE) != 0 && (p & StaticProperty.HAS_SIDE_EFFECTS) == 0
-                && ((getDependencies() & StaticProperty.DEPENDS_ON_ASSIGNABLE_GLOBALS) == 0);
+                && ((d & StaticProperty.DEPENDS_ON_ASSIGNABLE_GLOBALS) == 0)
+                && ((d & StaticProperty.DEPENDS_ON_POSITION) == 0)
+                && ((d & StaticProperty.DEPENDS_ON_LAST) == 0);
     }
 
     /**
@@ -393,12 +401,14 @@ public abstract class Expression implements IdentityComparable {
      */
 
     public void setRetainedStaticContext(RetainedStaticContext rsc) {
-        retainedStaticContext = rsc;
-        for (Operand o : operands()) {
-            if (o != null) {
-                Expression child = o.getChildExpression();
-                if (child != null && child.retainedStaticContext == null) {
-                    child.setRetainedStaticContext(rsc);
+        if (rsc != null) {
+            retainedStaticContext = rsc;
+            for (Operand o : operands()) {
+                if (o != null) {
+                    Expression child = o.getChildExpression();
+                    if (child != null && child.retainedStaticContext == null) {
+                        child.setRetainedStaticContext(rsc);
+                    }
                 }
             }
         }
@@ -413,22 +423,21 @@ public abstract class Expression implements IdentityComparable {
      */
 
     public void setRetainedStaticContextThoroughly(RetainedStaticContext rsc) {
-//        if (rsc != null) {
-//            System.err.println("setRSC " + toShortString() + " - " + rsc);
-//        }
-        retainedStaticContext = rsc;
-        for (Operand o : operands()) {
-            if (o != null) {
-                Expression child = o.getChildExpression();
-                if (child != null) {
-                    if (child.getLocalRetainedStaticContext() == null) {
-                        child.setRetainedStaticContextThoroughly(rsc);
-                    } else {
-                        rsc = child.getLocalRetainedStaticContext();
-                        for (Operand p : child.operands()) {
-                            Expression grandchild = p.getChildExpression();
-                            if (grandchild != null) {
-                                grandchild.setRetainedStaticContextThoroughly(rsc);
+        if (rsc != null) {
+            retainedStaticContext = rsc;
+            for (Operand o : operands()) {
+                if (o != null) {
+                    Expression child = o.getChildExpression();
+                    if (child != null) {
+                        if (child.getLocalRetainedStaticContext() == null) {
+                            child.setRetainedStaticContextThoroughly(rsc);
+                        } else {
+                            rsc = child.getLocalRetainedStaticContext();
+                            for (Operand p : child.operands()) {
+                                Expression grandchild = p.getChildExpression();
+                                if (grandchild != null) {
+                                    grandchild.setRetainedStaticContextThoroughly(rsc);
+                                }
                             }
                         }
                     }
@@ -551,7 +560,7 @@ public abstract class Expression implements IdentityComparable {
     /*@NotNull*/
     protected final void typeCheckChildren(ExpressionVisitor visitor,
                                 ContextItemStaticInfo contextInfo) throws XPathException {
-        for (Operand o : checkedOperands()) {
+        for (Operand o : operands()) {
             o.typeCheck(visitor, contextInfo);
         }
     }
@@ -1121,7 +1130,12 @@ public abstract class Expression implements IdentityComparable {
      */
 
     public final Location getLocation() {
-        return location;
+        if ((location == null || location == ExplicitLocation.UNKNOWN_LOCATION) && getParentExpression() != null) {
+            // Only go up one level to avoid risk of infinite recursion: see bug 3703#1
+            return getParentExpression().location;
+        } else {
+            return location;
+        }
     }
 
     /**
@@ -1592,6 +1606,22 @@ public abstract class Expression implements IdentityComparable {
         return System.identityHashCode(Expression.this.getLocation());
     }
 
+
+    public int hashCode() {
+        if (cachedHashCode == -1) {
+            cachedHashCode = computeHashCode();
+        }
+        return cachedHashCode;
+    }
+
+    public int computeHashCode() {
+        return super.hashCode();
+    }
+
+    public boolean isSameExpression(Expression other) {
+        return hashCode() == other.hashCode() && equals(other);
+    }
+
     /**
      * Set an extra property on the expression. (Used for Saxon-EE properties, e.g. properties
      * relating to streaming).
