@@ -171,9 +171,9 @@ public class IntegerRangeTest extends Expression {
      * Hashcode supporting equals()
      */
 
-    public int hashCode() {
-        int h = getValue().hashCode() + 77;
-        h ^= getMin().hashCode() ^ getMax().hashCode();
+    public int computeHashCode() {
+        int h = getValue().computeHashCode() + 77;
+        h ^= getMin().computeHashCode() ^ getMax().computeHashCode();
         return h;
     }
 
@@ -194,7 +194,7 @@ public class IntegerRangeTest extends Expression {
                     return BooleanValue.FALSE;
                 }
                 maxVal = (IntegerValue) getMax().evaluateItem(c);
-                if (maxVal == null || maxVal.compareTo(minVal) <= 0) {
+                if (maxVal == null || maxVal.compareTo(minVal) < 0) {
                     return BooleanValue.FALSE;
                 }
             }
