@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2017 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -11,15 +11,18 @@ import net.sf.saxon.Configuration;
 import net.sf.saxon.Controller;
 import net.sf.saxon.lib.ExternalObjectModel;
 import net.sf.saxon.lib.ParseOptions;
-import net.sf.saxon.ma.map.HashTrieMap;
+import net.sf.saxon.ma.arrays.ArrayItem;
+import net.sf.saxon.ma.arrays.ArrayItemType;
 import net.sf.saxon.ma.map.MapItem;
 import net.sf.saxon.ma.map.MapType;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.AnyNodeTest;
 import net.sf.saxon.pattern.NodeKindTest;
+import net.sf.saxon.s9api.*;
 import net.sf.saxon.trans.SaxonErrorCode;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.*;
+import net.sf.saxon.type.ItemType;
 import net.sf.saxon.value.*;
 
 import javax.xml.transform.Source;
@@ -44,8 +47,15 @@ public abstract class JPConverter {
     private static HashMap<Class, JPConverter> converterMap = new HashMap<Class, JPConverter>();
 
     static {
-        //converterMap.put(Object.class, FromObject.INSTANCE);
-        converterMap.put(SequenceIterator.class, new FromSequenceIterator());
+        converterMap.put(XdmValue.class, new FromXdmValue(AnyItemType.getInstance(), StaticProperty.ALLOWS_ZERO_OR_MORE));
+        converterMap.put(XdmItem.class, new FromXdmValue(AnyItemType.getInstance(), StaticProperty.ALLOWS_ONE));
+        converterMap.put(XdmAtomicValue.class, new FromXdmValue(BuiltInAtomicType.ANY_ATOMIC, StaticProperty.ALLOWS_ONE));
+        converterMap.put(XdmNode.class, new FromXdmValue(AnyNodeTest.getInstance(), StaticProperty.ALLOWS_ONE));
+        converterMap.put(XdmFunctionItem.class, new FromXdmValue(AnyFunctionType.getInstance(), StaticProperty.ALLOWS_ONE));
+        converterMap.put(XdmMap.class, new FromXdmValue(MapType.ANY_MAP_TYPE, StaticProperty.ALLOWS_ONE));
+        converterMap.put(XdmArray.class, new FromXdmValue(ArrayItemType.getInstance(), StaticProperty.ALLOWS_ONE));
+        converterMap.put(XdmEmptySequence.class, new FromXdmValue(ErrorType.getInstance(), StaticProperty.ALLOWS_ZERO));
+        converterMap.put(SequenceIterator.class, FromSequenceIterator.INSTANCE);
         converterMap.put(Sequence.class, FromSequence.INSTANCE);
         converterMap.put(OneOrMore.class, FromSequence.INSTANCE);
         converterMap.put(One.class, FromSequence.INSTANCE);
@@ -70,11 +80,9 @@ public abstract class JPConverter {
         converterMap.put(byte.class, FromByte.INSTANCE);
         converterMap.put(Character.class, FromCharacter.INSTANCE);
         converterMap.put(char.class, FromCharacter.INSTANCE);
-        //map.put(QName.class, new FromQName());
         converterMap.put(URI.class, FromURI.INSTANCE);
         converterMap.put(URL.class, FromURI.INSTANCE);
         converterMap.put(Date.class, FromDate.INSTANCE);
-        //map.put(Source.class, FromSource.INSTANCE);
         converterMap.put(long[].class, FromLongArray.INSTANCE);
         converterMap.put(int[].class, FromIntArray.INSTANCE);
         converterMap.put(short[].class, FromShortArray.INSTANCE);
@@ -84,10 +92,7 @@ public abstract class JPConverter {
         converterMap.put(float[].class, FromFloatArray.INSTANCE);
         converterMap.put(boolean[].class, FromBooleanArray.INSTANCE);
         converterMap.put(Collection.class, FromCollection.INSTANCE);
-//#if EE==true||PE==true
-        converterMap.put(Map.class, FromMap.INSTANCE);
-//#endif
-        //map.put(Object.class, FromExternalObject.INSTANCE);
+
     }
 
     private static Map<Class, ItemType> itemTypeMap = new HashMap<Class, ItemType>();
@@ -97,7 +102,7 @@ public abstract class JPConverter {
         itemTypeMap.put(StringValue.class, BuiltInAtomicType.STRING);
         itemTypeMap.put(DoubleValue.class, BuiltInAtomicType.DOUBLE);
         itemTypeMap.put(FloatValue.class, BuiltInAtomicType.FLOAT);
-        itemTypeMap.put(DecimalValue.class, BuiltInAtomicType.DECIMAL);
+        itemTypeMap.put(BigDecimalValue.class, BuiltInAtomicType.DECIMAL);
         itemTypeMap.put(IntegerValue.class, BuiltInAtomicType.INTEGER);
         itemTypeMap.put(DurationValue.class, BuiltInAtomicType.DURATION);
         itemTypeMap.put(DayTimeDurationValue.class, BuiltInAtomicType.DAY_TIME_DURATION);
@@ -117,9 +122,8 @@ public abstract class JPConverter {
         itemTypeMap.put(Base64BinaryValue.class, BuiltInAtomicType.BASE64_BINARY);
         itemTypeMap.put(NodeInfo.class, AnyNodeTest.getInstance());
         itemTypeMap.put(TreeInfo.class, NodeKindTest.DOCUMENT);
-//#if EE==true || PE==true
         itemTypeMap.put(MapItem.class, MapType.getInstance());
-//#endif
+        itemTypeMap.put(ArrayItem.class, ArrayItemType.getInstance());
         itemTypeMap.put(Function.class, AnyFunctionType.getInstance());
         itemTypeMap.put(AtomicValue.class, BuiltInAtomicType.ANY_ATOMIC);
         itemTypeMap.put(UntypedAtomicValue.class, BuiltInAtomicType.UNTYPED_ATOMIC);
@@ -133,6 +137,9 @@ public abstract class JPConverter {
         cardinalityMap.put(OneOrMore.class, StaticProperty.ALLOWS_ONE_OR_MORE);
         cardinalityMap.put(One.class, StaticProperty.EXACTLY_ONE);
         cardinalityMap.put(ZeroOrOne.class, StaticProperty.ALLOWS_ZERO_OR_ONE);
+        cardinalityMap.put(XdmValue.class, StaticProperty.ALLOWS_ZERO_OR_MORE);
+        cardinalityMap.put(XdmItem.class, StaticProperty.ALLOWS_ZERO_OR_MORE);
+        cardinalityMap.put(XdmEmptySequence.class, StaticProperty.ALLOWS_ZERO);
     }
 
     /**
@@ -151,6 +158,7 @@ public abstract class JPConverter {
         }
 
         if (Sequence.class.isAssignableFrom(javaClass)) {
+            // Following code caters for classes such as OneOrMore<BooleanValue>
             if (genericType instanceof ParameterizedType) {
                 java.lang.reflect.Type[] params = ((ParameterizedType)genericType).getActualTypeArguments();
                 if (params.length == 1 && params[0] instanceof Class && Item.class.isAssignableFrom((Class) params[0])) {
@@ -276,6 +284,30 @@ public abstract class JPConverter {
         }
     }
 
+    public static class FromXdmValue extends JPConverter {
+
+        private ItemType resultType;
+        private int cardinality;
+
+        public FromXdmValue(ItemType resultType, int cardinality) {
+            this.resultType = resultType;
+            this.cardinality = cardinality;
+        }
+
+        public Sequence convert(Object object, XPathContext context) throws XPathException {
+            return ((XdmValue)object).getUnderlyingValue();
+        }
+
+        public ItemType getItemType() {
+            return resultType;
+        }
+
+        public int getCardinality() {
+            return cardinality;
+        }
+
+    }
+
     public static class FromSequence extends JPConverter {
 
         public static final FromSequence INSTANCE =
@@ -358,8 +390,8 @@ public abstract class JPConverter {
     public static class FromBigDecimal extends JPConverter {
         public static final FromBigDecimal INSTANCE = new FromBigDecimal();
 
-        public DecimalValue convert(Object object, XPathContext context) throws XPathException {
-            return new DecimalValue((BigDecimal) object);
+        public BigDecimalValue convert(Object object, XPathContext context) throws XPathException {
+            return new BigDecimalValue((BigDecimal) object);
         }
 
         public ItemType getItemType() {
@@ -565,53 +597,6 @@ public abstract class JPConverter {
 
     }
 
-    //#if EE==true||PE==true
-    public static class FromMap extends JPConverter {
-
-        public static final FromMap INSTANCE = new FromMap();
-
-        public MapItem convert(Object object, XPathContext context) throws XPathException {
-            Map<Object, Object> jMap = (Map<Object, Object>) object;
-            final Configuration config = context.getConfiguration();
-            HashTrieMap pMap = new HashTrieMap(context);
-            for (Map.Entry<Object, Object> entry : jMap.entrySet()) {
-                Object key = entry.getKey();
-                Object value = entry.getValue();
-                JPConverter keyConverter = allocate(key.getClass(), null, config);
-                JPConverter valueConverter = allocate(value.getClass(), null, config);
-                try {
-                    Item keyItem = SequenceTool.asItem(keyConverter.convert(key, context));
-                    Sequence keyValue = valueConverter.convert(value, context);
-                    if (keyItem instanceof AtomicValue && keyValue != null) {
-                        pMap.initialPut((AtomicValue) keyItem, keyValue);
-                    }
-                } catch (XPathException e) {
-                    throw new XPathException(
-                            "Returned Map contains an object that cannot be converted to an Item ("
-                                    + key.getClass() + "): " + e.getMessage(),
-                            SaxonErrorCode.SXJE0051);
-                }
-            }
-            return pMap;
-        }
-
-        public ItemType getItemType() {
-            return AnyItemType.getInstance();
-        }
-
-        /**
-         * Get the cardinality of the XPath value that will result from the conversion
-         *
-         * @return the cardinality of the result
-         */
-
-        public int getCardinality() {
-            return StaticProperty.ALLOWS_ZERO_OR_MORE;
-        }
-
-    }
-//#endif
-
 
     public static class FromSource extends JPConverter {
 
@@ -705,13 +690,13 @@ public abstract class JPConverter {
         public Sequence convert(Object object, XPathContext context) throws XPathException {
             Item[] array = new Item[((byte[]) object).length];
             for (int i = 0; i < array.length; i++) {
-                array[i] = Int64Value.makeDerived(255 & (int) ((byte[]) object)[i], BuiltInAtomicType.BYTE);
+                array[i] = Int64Value.makeDerived(255 & (int) ((byte[]) object)[i], BuiltInAtomicType.UNSIGNED_BYTE);
             }
             return new SequenceExtent(array);
         }
 
         public ItemType getItemType() {
-            return BuiltInAtomicType.BYTE;
+            return BuiltInAtomicType.UNSIGNED_BYTE;
         }
 
         public int getCardinality() {
