@@ -492,11 +492,11 @@ public class Literal extends Expression {
      * Return a hash code to support the equals() function
      */
 
-    public int hashCode() {
+    public int computeHashCode() {
         if (value instanceof AtomicSequence) {
             return ((AtomicSequence) value).getSchemaComparable().hashCode();
         } else {
-            return super.hashCode();
+            return super.computeHashCode();
         }
     }
 
@@ -757,6 +757,21 @@ public class Literal extends Expression {
         }
     }
 
+    /**
+     * Make a literal, taking the retained static context and location information from another
+     * expression which is being simplified/optimized
+     * @param value the literal value
+     * @param origin the expression whose context and location information is to be retained
+     * @return the Literal
+     */
+
+    public static Literal makeLiteral(GroundedValue value, Expression origin) {
+        Literal lit = makeLiteral(value);
+        lit.setRetainedStaticContext(origin.getRetainedStaticContext());
+        ExpressionTool.copyLocationInfo(origin, lit);
+        return lit;
+    }
+
     /**
      * Get the (partial) name of a class that supports streaming of this kind of expression
      *
