@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2017 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,29 +7,25 @@
 
 package net.sf.saxon.expr;
 
-import com.saxonica.ee.bytecode.ExpressionCompiler;
-import com.saxonica.ee.bytecode.SlashExpressionCompiler;
-import com.saxonica.ee.stream.adjunct.ForEachAdjunct;
-import com.saxonica.functions.xslt3.CopyOfFn;
-import com.saxonica.functions.xslt3.SnapshotFn;
 import net.sf.saxon.Configuration;
 import net.sf.saxon.expr.instruct.CopyOf;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.expr.sort.DocumentSorter;
+import net.sf.saxon.functions.Doc;
+import net.sf.saxon.functions.DocumentFn;
 import net.sf.saxon.functions.KeyFn;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.*;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.XPathException;
-import net.sf.saxon.type.ErrorType;
-import net.sf.saxon.type.ItemType;
-import net.sf.saxon.type.Type;
-import net.sf.saxon.type.TypeHierarchy;
+import net.sf.saxon.type.*;
 import net.sf.saxon.value.Cardinality;
 import net.sf.saxon.value.IntegerValue;
 import net.sf.saxon.value.SequenceType;
 
-import java.util.*;
+import java.util.ArrayList;
+import java.util.List;
+import java.util.Stack;
 
 
 /**
@@ -132,6 +128,20 @@ public class SlashExpression extends BinaryExpression
         return getStep().getItemType();
     }
 
+
+    /**
+     * Get the static type of the expression as a UType, following precisely the type
+     * inference rules defined in the XSLT 3.0 specification.
+     *
+     * @return the static item type of the expression according to the XSLT 3.0 defined rules
+     * @param contextItemType
+     */
+    @Override
+    public UType getStaticUType(UType contextItemType) {
+        return getStep().getStaticUType(getStart().getStaticUType(contextItemType));
+    }
+
+
     /**
      * For an expression that returns an integer or a sequence of integers, get
      * a lower and upper bound on the values of the integers that may be returned, from
@@ -168,9 +178,11 @@ public class SlashExpression extends BinaryExpression
 
         // The first operand must be of type node()*
 
+        Configuration config = visitor.getConfiguration();
+        TypeChecker tc = config.getTypeChecker(false);
         RoleDiagnostic role0 = new RoleDiagnostic(RoleDiagnostic.BINARY_EXPR, "/", 0);
         role0.setErrorCode("XPTY0019");
-        setStart(TypeChecker.staticTypeCheck(getStart(), SequenceType.NODE_SEQUENCE, false, role0, visitor));
+        setStart(tc.staticTypeCheck(getStart(), SequenceType.NODE_SEQUENCE, role0, visitor));
 
         // Now check the second operand
 
@@ -180,7 +192,9 @@ public class SlashExpression extends BinaryExpression
             return Literal.makeEmptySequence();
         }
 
-        getRhs().typeCheck(visitor, new ContextItemStaticInfo(startType, false, getStart()));
+        ContextItemStaticInfo cit = config.makeContextItemStaticInfo(startType, false);
+        cit.setContextSettingExpression(getStart());
+        getRhs().typeCheck(visitor, cit);
 
         // If the expression has the form (a//descendant-or-self::node())/b, try to simplify it to
         // use the descendant axis
@@ -200,6 +214,11 @@ public class SlashExpression extends BinaryExpression
             return getStart();
         }
 
+        if (getStep() instanceof AxisExpression && ((AxisExpression)getStep()).getAxis() == AxisInfo.SELF &&
+                config.getTypeHierarchy().isSubType(startType, getStep().getItemType())) {
+            return getStart();
+        }
+
         return this;
     }
 
@@ -209,7 +228,7 @@ public class SlashExpression extends BinaryExpression
     // doesn't require sorting. Note that we can't do this until type information is available,
     // as we need to know whether any filters are positional or not.
 
-    private SlashExpression simplifyDescendantPath(StaticContext env) {
+    public SlashExpression simplifyDescendantPath(StaticContext env) {
 
         Expression underlyingStep = getStep();
         while (underlyingStep instanceof FilterExpression) {
@@ -294,7 +313,7 @@ public class SlashExpression extends BinaryExpression
 
             Expression newPath = ExpressionTool.makePathExpression(startPath.getStart(), newStep, false);
             if (!(newPath instanceof SlashExpression)) {
-                throw new AssertionError();
+                return null;
             }
             ExpressionTool.copyLocationInfo(this, newPath);
             return (SlashExpression) newPath;
@@ -321,11 +340,14 @@ public class SlashExpression extends BinaryExpression
 
     /*@NotNull*/
     public Expression optimize(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
-        TypeHierarchy th = getConfiguration().getTypeHierarchy();
-        Optimizer opt = getConfiguration().obtainOptimizer();
+        Configuration config = visitor.getConfiguration();
+        TypeHierarchy th = config.getTypeHierarchy();
+        Optimizer opt = config.obtainOptimizer();
 
         getLhs().optimize(visitor, contextItemType);
-        getRhs().optimize(visitor, new ContextItemStaticInfo(getStart().getItemType(), false, getStart()));
+        ContextItemStaticInfo cit = visitor.getConfiguration().makeContextItemStaticInfo(getStart().getItemType(), false);
+        cit.setContextSettingExpression(getStart());
+        getRhs().optimize(visitor, cit);
 
         if (Literal.isEmptySequence(getStart()) || Literal.isEmptySequence(getStep())) {
             return Literal.makeEmptySequence();
@@ -339,13 +361,17 @@ public class SlashExpression extends BinaryExpression
 
         // Rewrite a/b[filter] as (a/b)[filter] to improve the chance of indexing
 
-        Expression lastStep = getLastStep();
-        if (lastStep instanceof FilterExpression && !((FilterExpression) lastStep).isPositional(th)) {
-            Expression leading = getLeadingSteps();
-            Expression p2 = ExpressionTool.makePathExpression(leading, ((FilterExpression) lastStep).getSelectExpression(), false);
-            Expression f2 = new FilterExpression(p2, ((FilterExpression) lastStep).getFilter());
-            ExpressionTool.copyLocationInfo(this, f2);
-            return f2.optimize(visitor, contextItemType);
+        Expression firstStep = getFirstStep();
+        if (!(firstStep.isCallOn(Doc.class) || firstStep.isCallOn(DocumentFn.class))) {
+            // Avoid the rewrite if the path starts with doc() for streaming reasons
+            Expression lastStep = getLastStep();
+            if (lastStep instanceof FilterExpression && !((FilterExpression) lastStep).isPositional(th)) {
+                Expression leading = getLeadingSteps();
+                Expression p2 = ExpressionTool.makePathExpression(leading, ((FilterExpression) lastStep).getSelectExpression(), false);
+                Expression f2 = new FilterExpression(p2, ((FilterExpression) lastStep).getFilter());
+                ExpressionTool.copyLocationInfo(this, f2);
+                return f2.optimize(visitor, contextItemType);
+            }
         }
 
         if (!visitor.isOptimizeForStreaming()) {
@@ -425,6 +451,9 @@ public class SlashExpression extends BinaryExpression
                 RootExpression root = new RootExpression();
                 ExpressionTool.copyLocationInfo(this, root);
                 Expression path = ExpressionTool.makePathExpression(root, this.copy(new RebindingMap()), false);
+                if (!(path instanceof SlashExpression)) {
+                    return null;
+                }
                 ExpressionTool.copyLocationInfo(this, path);
                 return (SlashExpression) path;
             }
@@ -456,15 +485,19 @@ public class SlashExpression extends BinaryExpression
      */
     @Override
     public int getCost() {
-        return getLhsExpression().getCost() * getRhsExpression().getCost();
+        int lh = getLhsExpression().getCost() + 1;
+        int rh = getRhsExpression().getCost();
+        double product = (double)lh * (double)rh;
+        return product > 1e7 ? 10000000 : Math.max(lh * rh, 10000000);
     }
 
     public Expression tryToMakeSorted(ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
         // Replace //x/y by descendant::y[parent::x] to eliminate the need for sorting
         // into document order, and to make the expression streamable
 
-        TypeHierarchy th = getConfiguration().getTypeHierarchy();
-        Optimizer opt = getConfiguration().obtainOptimizer();
+        Configuration config = visitor.getConfiguration();
+        TypeHierarchy th = config.getTypeHierarchy();
+        Optimizer opt = config.obtainOptimizer();
         Expression s1 = ExpressionTool.unfilteredExpression(getStart(), false);
         if (!(s1 instanceof AxisExpression && ((AxisExpression) s1).getAxis() == AxisInfo.DESCENDANT)) {
             return null;
@@ -529,34 +562,23 @@ public class SlashExpression extends BinaryExpression
     protected Expression promoteFocusIndependentSubexpressions(
             ExpressionVisitor visitor, ContextItemStaticInfo contextItemType) throws XPathException {
 
-        Optimizer opt = getConfiguration().obtainOptimizer();
-
-        PromotionOffer offer = new PromotionOffer(opt);
-        offer.action = PromotionOffer.FOCUS_INDEPENDENT;
-        offer.promoteDocumentDependent = (getStart().getSpecialProperties() & StaticProperty.CONTEXT_DOCUMENT_NODESET) != 0;
-        offer.containingExpression = this;
-
-        setStep(doPromotion(getStep(), offer));
-        ExpressionTool.resetStaticProperties(this);
-        if (offer.containingExpression != this) {
-            offer.containingExpression =
-                    offer.containingExpression.typeCheck(visitor, contextItemType).optimize(visitor, contextItemType);
-            return offer.containingExpression;
-        }
+//        Optimizer opt = getConfiguration().obtainOptimizer();
+//
+//        PromotionOffer offer = new PromotionOffer(opt);
+//        offer.action = PromotionOffer.FOCUS_INDEPENDENT;
+//        offer.promoteDocumentDependent = (getStart().getSpecialProperties() & StaticProperty.CONTEXT_DOCUMENT_NODESET) != 0;
+//        offer.containingExpression = this;
+//
+//        setStep(doPromotion(getStep(), offer));
+//        ExpressionTool.resetStaticProperties(this);
+//        if (offer.containingExpression != this) {
+//            offer.containingExpression =
+//                    offer.containingExpression.typeCheck(visitor, contextItemType).optimize(visitor, contextItemType);
+//            return offer.containingExpression;
+//        }
         return this;
     }
 
-    /**
-     * Promote this expression if possible
-     */
-
-    public void promoteChildren(PromotionOffer offer) throws XPathException {
-        setStart(doPromotion(getStart(), offer));
-        // Don't pass on requests to the step. We could pass them on, but only after augmenting
-            // them to say we are interested in subexpressions that don't depend on either the
-            // outer context or the inner context.
-    }
-
     /**
      * Replace this expression by a simpler expression that delivers the results without regard
      * to order.
@@ -625,8 +647,8 @@ public class SlashExpression extends BinaryExpression
      */
 
     /*@NotNull*/
-    public SlashExpression copy(RebindingMap rebindings) {
-        SlashExpression exp = (SlashExpression)ExpressionTool.makePathExpression(getStart().copy(rebindings), getStep().copy(rebindings), false);
+    public Expression copy(RebindingMap rebindings) {
+        Expression exp = ExpressionTool.makePathExpression(getStart().copy(rebindings), getStep().copy(rebindings), false);
         ExpressionTool.copyLocationInfo(this, exp);
         return exp;
     }
@@ -638,14 +660,16 @@ public class SlashExpression extends BinaryExpression
      */
 
     public int computeSpecialProperties() {
-//#ifdefined STREAM
-        if (getStep().isCallOn(CopyOfFn.class) || getStep().isCallOn(SnapshotFn.class)) {
-            return StaticProperty.ORDERED_NODESET | StaticProperty.PEER_NODESET | StaticProperty.NON_CREATIVE;
-        }
-//#endif
+
         int startProperties = getStart().getSpecialProperties();
         int stepProperties = getStep().getSpecialProperties();
 
+        if ((stepProperties & StaticProperty.ALL_NODES_NEWLY_CREATED) != 0) {
+            // Deem copies/snapshots to be in document order
+            return StaticProperty.ORDERED_NODESET | StaticProperty.PEER_NODESET | StaticProperty.NON_CREATIVE;
+        }
+
+
 //        System.err.println("====" + toShortString() + "====");
 //        System.err.println("START: " + StaticProperty.display(startProperties));
 //        System.err.println("STEP: " + StaticProperty.display(stepProperties));
@@ -737,7 +761,7 @@ public class SlashExpression extends BinaryExpression
 
         // The result is sorted if the step is creative (e.g. a call to copy-of())
 
-        if ((stepProperties & StaticProperty.NON_CREATIVE) == 0) {
+        if ((stepProperties & StaticProperty.ALL_NODES_NEWLY_CREATED) != 0) {
             return true;
         }
 
@@ -858,8 +882,8 @@ public class SlashExpression extends BinaryExpression
      * get HashCode for comparing two expressions
      */
 
-    public int hashCode() {
-        return "SlashExpression".hashCode() + getStart().hashCode() + getStep().hashCode();
+    public int computeHashCode() {
+        return "SlashExpression".hashCode() + getStart().computeHashCode() + getStep().computeHashCode();
     }
 
     /**
@@ -919,7 +943,7 @@ public class SlashExpression extends BinaryExpression
 
     @Override
     public String toShortString() {
-        return getStart().toShortString() + "/" +  getStep().toShortString();
+        return ExpressionTool.parenthesizeShort(getStart()) + "/" + ExpressionTool.parenthesizeShort(getStep());
     }
 
     /**
@@ -1044,33 +1068,15 @@ public class SlashExpression extends BinaryExpression
     }
 
 
-//#ifdefined STREAM
-
     /**
-     * Get a class that supports streamed evaluation of this expression
+     * Get the (partial) name of a class that supports streaming of this kind of expression
      *
-     * @return the relevant StreamingAdjunct, or null if none is available
+     * @return the partial name of a class that can be instantiated to provide streaming support in Saxon-EE,
+     * or null if there is no such class
      */
     @Override
-    public ForEachAdjunct getStreamingAdjunct() {
-        return new ForEachAdjunct(); //sic
+    public String getStreamerName() {
+        return "ForEach"; // sic
     }
-
-//#endif
-//#ifdefined BYTECODE
-
-    /**
-     * Return the compiler of the Slash expression
-     *
-     * @return the relevant ExpressionCompiler
-     */
-    @Override
-    public ExpressionCompiler getExpressionCompiler() {
-        return new SlashExpressionCompiler();
-    }
-//#endif
-
-
-
 }
 
