@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2017 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -7,20 +7,18 @@
 
 package net.sf.saxon.expr.instruct;
 
-import com.saxonica.ee.stream.Posture;
-import com.saxonica.ee.stream.PostureAndSweep;
-import com.saxonica.ee.stream.TemplateInversion;
 import net.sf.saxon.expr.*;
-import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.Location;
 import net.sf.saxon.expr.parser.RebindingMap;
 import net.sf.saxon.om.StandardNames;
+import net.sf.saxon.om.StructuredQName;
 import net.sf.saxon.pattern.Pattern;
+import net.sf.saxon.style.Compilation;
+import net.sf.saxon.style.ComponentDeclaration;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trace.LocationKind;
-import net.sf.saxon.trans.Rule;
-import net.sf.saxon.trans.RuleTarget;
-import net.sf.saxon.trans.SaxonErrorCode;
+import net.sf.saxon.trans.rules.Rule;
+import net.sf.saxon.trans.rules.RuleTarget;
 import net.sf.saxon.trans.XPathException;
 import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.ItemType;
@@ -37,31 +35,32 @@ import java.util.List;
  * only in the Rule object that references this Template. This allows two rules to share the same template,
  * with different precedences. This occurs when a stylesheet module is imported more than once, from different
  * places, with different import precedences.
- *
+ * <p/>
  * <p>From Saxon 9.7, the old Template class is split into NamedTemplate and TemplateRule.</p>
+ * <p>From Saxon 9.8, a subclass is used in Saxon-EE</p>
  */
 
-public class TemplateRule implements RuleTarget, Location {
+public class TemplateRule implements RuleTarget, Location, ExpressionOwner {
 
     // The body of the template is represented by an expression,
     // which is responsible for any type checking that's needed.
 
-    private Expression body;
-    private Pattern matchPattern;
+    protected Expression body;
+    protected Pattern matchPattern;
     private boolean hasRequiredParams;
     private boolean bodyIsTailCallReturner;
     private SequenceType requiredType;
     private boolean declaredStreamable;
     private ItemType requiredContextItemType = AnyItemType.getInstance();
-    private boolean mayOmitContextItem = true;
-    private boolean maySupplyContextItem = true;
+    private boolean absentFocus;
     private SlotManager stackFrameMap;
     private PackageData packageData;
     private String systemId;
     private int lineNumber;
 
+
     private List<Rule> rules = new ArrayList<Rule>();
-    private List<TemplateRule> slaveCopies = new ArrayList<TemplateRule>();
+    protected List<TemplateRule> slaveCopies = new ArrayList<TemplateRule>();
 
     /**
      * Create a template
@@ -93,15 +92,13 @@ public class TemplateRule implements RuleTarget, Location {
     /**
      * Set the required context item type. Used when there is an xsl:context-item child element
      *
-     * @param type          the required context item type
-     * @param mayBeOmitted  true if the context item may be absent
-     * @param mayBeSupplied true if the context item may be supplied
+     * @param type        the required context item type
+     * @param absentFocus true if the context item is treated as absent even if supplied (use="absent")
      */
 
-    public void setContextItemRequirements(ItemType type, boolean mayBeOmitted, boolean mayBeSupplied) {
+    public void setContextItemRequirements(ItemType type, boolean absentFocus) {
         requiredContextItemType = type;
-        mayOmitContextItem = mayBeOmitted;
-        maySupplyContextItem = mayBeSupplied;
+        this.absentFocus = absentFocus;
     }
 
     public int getComponentKind() {
@@ -203,6 +200,7 @@ public class TemplateRule implements RuleTarget, Location {
     /**
      * Get the rules that use this template. For a template with no match pattern, this will be
      * an empty list. For a union pattern, there will be one rule for each branch of the union.
+     *
      * @return the rules corresponding to this template.
      */
 
@@ -210,29 +208,6 @@ public class TemplateRule implements RuleTarget, Location {
         return rules;
     }
 
-    /**
-     * Say whether or not this template is declared as streamable
-     *
-     * @param streamable true if the template belongs to a streamable mode; set to false if it does not belong
-     *                   to a streamable mode, or if it is found that the template is not actually streamable, and fallback
-     *                   to non-streaming has been requested.
-     */
-
-    public void setDeclaredStreamable(boolean streamable) {
-        this.declaredStreamable = streamable;
-    }
-
-    /**
-     * Ask whether or not this template is declared as streamable
-     *
-     * @return true if the template belongs to a streamable mode; false if it does not belong
-     * to a streamable mode, or if it is found that the template is not actually streamable, and fallback
-     * to non-streaming has been requested.
-     */
-
-    public boolean isDeclaredStreamable() {
-        return declaredStreamable;
-    }
 
     public int getContainerGranularity() {
         return 0;
@@ -283,12 +258,8 @@ public class TemplateRule implements RuleTarget, Location {
         return requiredContextItemType;
     }
 
-    public boolean isMayOmitContextItem() {
-        return mayOmitContextItem;
-    }
-
-    public boolean isMaySupplyContextItem() {
-        return maySupplyContextItem;
+    public boolean isAbsentFocus() {
+        return absentFocus;
     }
 
 
@@ -299,8 +270,8 @@ public class TemplateRule implements RuleTarget, Location {
     }
 
     private static void gatherLocalParams(Expression exp, List<LocalParam> result) {
-        if (exp instanceof LocalParamSetter) {
-            result.add(((LocalParamSetter) exp).getBinding());
+        if (exp instanceof LocalParam) {
+            result.add((LocalParam) exp);
         } else {
             for (Operand o : exp.operands()) {
                 gatherLocalParams(o.getChildExpression(), result);
@@ -308,13 +279,33 @@ public class TemplateRule implements RuleTarget, Location {
         }
     }
 
+    /**
+     * Prepare for JIT compilation.
+     *
+     * @param compilation the XSLT compilation
+     * @param decl        the component declaration of this template rule
+     */
+
+    public void prepareInitializer(Compilation compilation, ComponentDeclaration decl, StructuredQName modeName) {
+        // No action in Saxon-HE
+    }
+
+    /**
+     * Ensure that any first-time initialization has been done. Used in Saxon-EE
+     * to do JIT compilation
+     */
+
+    public void initialize() throws XPathException {
+        // No action needed in Saxon-HE
+    }
+
     /**
      * Process the template, without returning any tail calls. This path is used by
      * xsl:apply-imports and xsl:next-match
      *
      * @param context The dynamic context, giving access to the current node,
      * @throws XPathException if a dynamic error occurs while evaluating
-     *                                           the template
+     *                        the template
      */
 
     public void apply(XPathContextMajor context) throws XPathException {
@@ -332,21 +323,21 @@ public class TemplateRule implements RuleTarget, Location {
      * @return null if the template exited normally; but if it was a tail call, details of the call
      * that hasn't been made yet and needs to be made by the caller
      * @throws XPathException if a dynamic error occurs while evaluating
-     *                                           the template
+     *                        the template
      */
 
     public TailCall applyLeavingTail(XPathContext context) throws XPathException {
-
+        //initialize();
         if (requiredContextItemType != AnyItemType.getInstance() &&
                 !requiredContextItemType.matches(context.getContextItem(), context.getConfiguration().getTypeHierarchy())) {
             XPathException err = new XPathException("The template requires a context item of type " + requiredContextItemType +
-                    ", but the supplied context item has type " +
-                    Type.getItemType(context.getContextItem(), context.getConfiguration().getTypeHierarchy()), "XTTE0590");
+                                                            ", but the supplied context item has type " +
+                                                            Type.getItemType(context.getContextItem(), context.getConfiguration().getTypeHierarchy()), "XTTE0590");
             err.setLocation(this);
             err.setIsTypeError(true);
             throw err;
         }
-        if (!maySupplyContextItem) {
+        if (absentFocus) {
             context = context.newMinorContext();
             context.setCurrentIterator(null);
         }
@@ -384,40 +375,60 @@ public class TemplateRule implements RuleTarget, Location {
      * Output diagnostic explanation to an ExpressionPresenter
      */
 
-    public void export(ExpressionPresenter presenter) throws XPathException{
-        presenter.startElement("template");
-
-        explainProperties(presenter);
-
-        presenter.emitAttribute("slots", "" + getStackFrameMap().getNumberOfVariables());
-        //presenter.emitAttribute("binds", "" + getDeclaringComponent().getComponentBindings().size());
-
-        if (matchPattern != null) {
-            presenter.setChildRole("match");
-            matchPattern.export(presenter);
-        }
-        if (getBody() != null) {
-            presenter.setChildRole("body");
-            getBody().export(presenter);
-        }
-        presenter.endElement();
+    public void export(ExpressionPresenter presenter) throws XPathException {
+        // NOT USED - see Rule.export
+        throw new UnsupportedOperationException();
+//        presenter.startElement("template");
+//
+//        explainProperties(presenter);
+//
+//        presenter.emitAttribute("slots", "" + getStackFrameMap().getNumberOfVariables());
+//        //presenter.emitAttribute("binds", "" + getDeclaringComponent().getComponentBindings().size());
+//
+//        if (matchPattern != null) {
+//            presenter.setChildRole("match");
+//            matchPattern.export(presenter);
+//        }
+//        if (getBody() != null) {
+//            presenter.setChildRole("body");
+//            getBody().export(presenter);
+//        }
+//        presenter.endElement();
     }
 
-    public void explainProperties(ExpressionPresenter presenter) {
+    /**
+     * Say whether or not this template is declared as streamable
+     *
+     * @param streamable true if the template belongs to a streamable mode; set to false if it does not belong
+     *                   to a streamable mode, or if it is found that the template is not actually streamable, and fallback
+     *                   to non-streaming has been requested.
+     */
+
+    public void setDeclaredStreamable(boolean streamable) {
+        // Overridden in Saxon-EE
+    }
+
+    public boolean isDeclaredStreamable() {
+        // Overridden in Saxon-EE
+        return false;
+    }
+
+    public void explainProperties(ExpressionPresenter presenter) throws XPathException {
         if (getRequiredContextItemType() != AnyItemType.getInstance()) {
             presenter.emitAttribute("cxt", getRequiredContextItemType().toString());
+            if ("JS".equals(presenter.getOption("target"))) {
+                int targetVersion = presenter.getIntOption("targetVersion", 1);
+                presenter.emitAttribute("jsTest", getRequiredContextItemType().generateJavaScriptItemTypeTest(AnyItemType.getInstance(), targetVersion));
+            }
         }
 
         String flags = "";
-        if (mayOmitContextItem) {
-            flags = "o";
-        }
-        if (maySupplyContextItem) {
+        if (!absentFocus) {
             flags += "s";
         }
         presenter.emitAttribute("flags", flags);
         if (getRequiredType() != SequenceType.ANY_SEQUENCE) {
-            presenter.emitAttribute("as", getRequiredType().toString());
+            presenter.emitAttribute("as", getRequiredType().toExportString());
         }
         presenter.emitAttribute("line", getLineNumber() + "");
         presenter.emitAttribute("module", getSystemId());
@@ -431,7 +442,6 @@ public class TemplateRule implements RuleTarget, Location {
     }
 
 
-
     /**
      * Create a copy of a template rule. This is needed when copying a rule from the "omniMode" (mode=#all)
      * to a specific mode. Because we want the rules to be chained in the right order within the mode object,
@@ -461,7 +471,7 @@ public class TemplateRule implements RuleTarget, Location {
         }
     }
 
-    private void copyTo(TemplateRule tr) {
+    protected void copyTo(TemplateRule tr) {
         if (body != null) {
             tr.body = body.copy(new RebindingMap());
         }
@@ -473,80 +483,17 @@ public class TemplateRule implements RuleTarget, Location {
         tr.requiredType = requiredType;
         tr.declaredStreamable = declaredStreamable; // ? this can vary from one mode to another
         tr.requiredContextItemType = requiredContextItemType;
-        tr.mayOmitContextItem = mayOmitContextItem;
-        tr.maySupplyContextItem = maySupplyContextItem;
+        tr.absentFocus = absentFocus;
         tr.stackFrameMap = stackFrameMap;
         tr.packageData = packageData;
         tr.systemId = systemId;
         tr.lineNumber = lineNumber;
     }
 
-//#ifdefined STREAM
-
-    /**
-     * Ask whether this template satisfies the rules for a guaranteed streamable template
-     *
-     * @param allowExtensions if true, the method tests for streamability against the Saxon
-     *                        rules rather than the W3C rules
-     * @param reasons         the caller may supply a list which on return will contain explanations
-     *                        for why the template is not streamable, suitable for inclusion in error messages
-     * @return true if the template is within the (Saxon or W3C) definition of guaranteed streamable
-     */
-
-    public boolean isActuallyStreamable(boolean allowExtensions, List<String> reasons) {
-        if (matchPattern == null) {
-            if (reasons != null) {
-                reasons.add("There is no match pattern");
-            }
-            return false;
-        }
-        if (!matchPattern.isMotionless(allowExtensions)) {
-            if (reasons != null) {
-                reasons.add("The match pattern is not motionless");
-            }
-            return false;
-        }
-        if (body == null) {
-            if (reasons != null) {
-                reasons.add("The template has no body");
-            }
-            return false;
-        }
-        ContextItemStaticInfo info = new ContextItemStaticInfo(matchPattern.getItemType(), false, true);
-        PostureAndSweep s = body.getStreamability(allowExtensions, info, reasons);
-        return s.getPosture() == Posture.GROUNDED;
+    public void setChildExpression(Expression expr) {
+        setBody(expr);
     }
 
 
-
-    /**
-     * Generate the inversion of the expression comprising the body of a template rules.
-     * Supported in Saxon-EE only
-     *
-     */
-
-
-    public void makeInversion() throws XPathException {
-        // Create the Jackson inversion of the template
-        if (inversion == null) {
-            if (getBody() instanceof TraceExpression) {
-                throw new XPathException(
-                    "Saxon restriction: code that uses streaming cannot be compiled with tracing enabled", SaxonErrorCode.SXST0065, this);
-            }
-            TemplateInversion inv = new TemplateInversion(this);
-            inv.setStackFrame(getStackFrameMap());
-            inv.invert();
-            inversion = inv;
-        }
-    }
-
-    private TemplateInversion inversion;
-
-    public TemplateInversion getInversion() {
-        return inversion;
-    }
-
-//#endif
-
 }
 
