@@ -60,9 +60,8 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
     private boolean isUpdating = false;
     private int inlineable = -1; // 0:no 1:yes -1:don't know
     private boolean overrideExtensionFunction = true;
-    private AnnotationList annotations;
+    private AnnotationList annotations = AnnotationList.EMPTY;
     private FunctionStreamability declaredStreamability = FunctionStreamability.UNCLASSIFIED;
-    private Controller preallocatedController = null;
     private Determinism determinism = Determinism.PROACTIVE;
     private int refCount = 0;
 
@@ -136,7 +135,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
             UserFunctionParameter ufp = parameterDefinitions[i];
             argTypes[i] = ufp.getRequiredType();
         }
-        return new SpecificFunctionType(argTypes, resultType);
+        return new SpecificFunctionType(argTypes, resultType, annotations);
     }
 
     /**
@@ -203,18 +202,6 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
         this.overrideExtensionFunction = overrideExtensionFunction;
     }
 
-
-    /**
-     * Supply the controller to be used when evaluating this function. This is used when the function is dynamically loaded
-     * when using the load-xquery-module function.
-     *
-     * @param controller the controller to be used (which may differ from the controller used by the caller)
-     */
-
-    public void setPreallocatedController(Controller controller) {
-        preallocatedController = controller;
-    }
-
     /**
      * Supply a set of annotations
      *
@@ -557,11 +544,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
 
     public XPathContextMajor makeNewContext(XPathContext oldContext) {
         XPathContextMajor c2;
-        if (preallocatedController == null) {
-            c2 = oldContext.newCleanContext();
-        } else {
-            c2 = preallocatedController.newXPathContext();
-        }
+        c2 = oldContext.newCleanContext();
         c2.setOrigin(this);
         c2.setReceiver(oldContext.getReceiver());
         c2.setTemporaryOutputState(StandardNames.XSL_FUNCTION);
@@ -717,7 +700,7 @@ public class UserFunction extends Actor implements Function, ContextOriginator {
         } else {
             flags += "d";
         }
-        if (isMemoFunction()) {
+        if (isMemoFunction() && !"JS".equals(presenter.getOption("target"))) {
             flags += "m";
         }
         switch (declaredStreamability) {
