@@ -44,6 +44,7 @@ import java.io.File;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.util.ArrayList;
+import java.util.Iterator;
 import java.util.List;
 
 /**
@@ -1222,6 +1223,9 @@ public class ExpressionTool {
             body = body.optimize(visitor, cit);
             body.setParentExpression(null);
             if (opt.isOptionSet(OptimizerOptions.LOOP_LIFTING)) {
+                if (opt.tracing) {
+                    body.verifyParentPointers();
+                }
                 body = LoopLifter.process(body, visitor, cit);
             }
             if (extractGlobals && compilation != null) {
@@ -1597,14 +1601,6 @@ public class ExpressionTool {
     public static Expression makePathExpression(
             /*@NotNull*/ Expression start, /*@NotNull*/ Expression step, boolean sortAndDeduplicate) {
 
-//        if (sortAndDeduplicate) {
-//            // if the final expression is to be sorted and deduplicated, then there is no need to apply this process
-//            // to the results of subexpressions
-        // code removed 2011-11-01 MHK - causes test expr36 to fail
-//            start = removeSorting(start);
-//            step = removeSorting(step);
-//        }
-
         // the expression /.. is sometimes used to represent the empty node-set. Applying this simplification
         // now avoids generating warnings for this case.
         if (start instanceof RootExpression &&
@@ -1964,13 +1960,30 @@ public class ExpressionTool {
      */
 
     public static String parenthesize(Expression exp) {
-        if (exp.operands().iterator().hasNext()) {
+        if (hasTwoOrMoreOperands(exp)) {
             return "(" + exp.toString() + ")";
         } else {
             return exp.toString();
         }
     }
 
+    public static String parenthesizeShort(Expression exp) {
+        if (hasTwoOrMoreOperands(exp)) {
+            return "(" + exp.toShortString() + ")";
+        } else {
+            return exp.toShortString();
+        }
+    }
+
+    private static boolean hasTwoOrMoreOperands(Expression exp) {
+        Iterator ops = exp.operands().iterator();
+        if (!ops.hasNext()) {
+            return false;
+        }
+        ops.next();
+        return ops.hasNext();
+    }
+
     public static void validateTree(Expression exp) {
         try {
             for (Operand o : exp.checkedOperands()) {
