@@ -8,7 +8,6 @@
 package net.sf.saxon.functions;
 
 import net.sf.saxon.expr.*;
-import net.sf.saxon.expr.instruct.UserFunction;
 import net.sf.saxon.expr.parser.ContextItemStaticInfo;
 import net.sf.saxon.expr.parser.ExpressionVisitor;
 import net.sf.saxon.expr.parser.RetainedStaticContext;
@@ -74,7 +73,7 @@ public abstract class SystemFunction extends AbstractFunction {
         SystemFunction fn = rsc.getConfiguration().makeSystemFunction(name, arity);
         if (fn == null) {
             rsc.getConfiguration().makeSystemFunction(name, arity);
-            throw new IllegalStateException(name);
+            throw new IllegalStateException(name + '#' + arity);
         }
         fn.setRetainedStaticContext(rsc);
         return fn;
@@ -449,10 +448,7 @@ public abstract class SystemFunction extends AbstractFunction {
      */
 
     public static Sequence dynamicCall(Function f, XPathContext context, Sequence[] args) throws XPathException {
-        if (f instanceof UserFunction) {
-            context = ((UserFunction)f).makeNewContext(context);
-        }
-        return f.call(context, args);
+        return f.call(f.makeNewContext(context), args);
     }
 
 
