@@ -203,7 +203,9 @@ public class SerializerFactory {
             props = props2;
             CharacterMap characterMap = ph.getCharacterMap();
             if (characterMap != null) {
-                CharacterMapIndex index = new CharacterMapIndex();
+                if (charMapIndex == null) {
+                    charMapIndex = new CharacterMapIndex();
+                }
                 charMapIndex.putCharacterMap(characterMap.getName(), characterMap);
                 props.setProperty(SaxonOutputKeys.USE_CHARACTER_MAPS, characterMap.getName().getClarkName());
             }
@@ -808,7 +810,12 @@ public class SerializerFactory {
      */
 
     protected ProxyReceiver newXHTMLIndenter(Receiver next, Properties outputProperties) {
-        return new HTMLIndenter(next, "xhtml");
+        String method = "xhtml";
+        String htmlVersion = outputProperties.getProperty("html-version");
+        if (htmlVersion != null && htmlVersion.startsWith("5")) {
+            method = "xhtml5";
+        }
+        return new HTMLIndenter(next, method);
     }
 
     /**
