@@ -274,6 +274,17 @@ public class RangeKey implements MapItem {
         return 1;
     }
 
+    /**
+     * Prepare an XPathContext object for evaluating the function
+     *
+     * @param callingContext the XPathContext of the function calling expression
+     * @return a suitable context for evaluating the function (which may or may
+     * not be the same as the caller's context)
+     */
+    public XPathContext makeNewContext(XPathContext callingContext) {
+        return callingContext;
+    }
+
     /**
      * Invoke the function
      *
