@@ -177,7 +177,7 @@ public class GeneralPositionalPattern extends Pattern {
 
     public int getDependencies() {
         // the only dependency that's interesting is a dependency on local variables
-        return positionExpr.getDependencies() & StaticProperty.DEPENDS_ON_LOCAL_VARIABLES;
+        return positionExpr.getDependencies() & (StaticProperty.DEPENDS_ON_LOCAL_VARIABLES | StaticProperty.DEPENDS_ON_USER_FUNCTIONS);
     }
 
     /**
@@ -255,15 +255,7 @@ public class GeneralPositionalPattern extends Pattern {
         } catch (XPathException.Circularity e) {
             throw e;
         } catch (XPathException e) {
-            if ("XTDE0640".equals(e.getErrorCodeLocalPart())) {
-                // Treat circularity error as fatal (test error213)
-                throw e;
-            }
-            XPathException err = new XPathException("An error occurred matching pattern {" + toString() + "}: ", e);
-            err.setXPathContext(c2);
-            err.setErrorCodeQName(e.getErrorCodeQName());
-            err.setLocation(getLocation());
-            c2.getController().recoverableError(err);
+            handleDynamicError(e, c2);
             return false;
         }
     }
@@ -325,7 +317,7 @@ public class GeneralPositionalPattern extends Pattern {
      * hashcode supporting equals()
      */
 
-    public int hashCode() {
+    public int computeHashCode() {
         return nodeTest.hashCode() ^ positionExpr.hashCode();
     }
 
