@@ -32,6 +32,7 @@ import net.sf.saxon.type.UType;
 public abstract class Pattern extends PseudoExpression {
 
     private double priority = 0.5;
+    private boolean recoverable = true;
 
     /**
      * Static factory method to make a Pattern by parsing a String. <br>
@@ -127,6 +128,32 @@ public abstract class Pattern extends PseudoExpression {
         /*originalText = text;*/
     }
 
+
+    public boolean isRecoverable() {
+        return recoverable;
+    }
+
+    public void setRecoverable(boolean recoverable) {
+        this.recoverable = recoverable;
+    }
+
+    protected void handleDynamicError(XPathException ex, XPathContext context) throws XPathException {
+        if ("XTDE0640".equals(ex.getErrorCodeLocalPart())) {
+            // Treat circularity error as fatal (test error213)
+            throw ex;
+        }
+        if (!isRecoverable()) {
+            // Typically happens when this is a pseudo-pattern used for scannable expressions when streaming
+            throw ex;
+        }
+        XPathException err = new XPathException("An error occurred matching pattern {" + toString() + "}: ", ex);
+        err.setXPathContext(context);
+        err.setErrorCodeQName(ex.getErrorCodeQName());
+        err.setLocation(getLocation());
+        context.getController().recoverableError(err);
+    }
+
+
     /**
      * Simplify the pattern by applying any context-independent optimisations.
      * Default implementation does nothing.
@@ -152,9 +179,9 @@ public abstract class Pattern extends PseudoExpression {
     }
 
     /**
-     * Get the dependencies of the pattern. The only possible dependency for a pattern is
-     * on local variables. This is analyzed in those patterns where local variables may appear.
-     *
+     * Get the dependencies of the pattern. The only interesting dependencies for a pattern are
+     * on local variables and user-defined functions. This is analyzed in those patterns where
+     * local variables may appear.
      * @return the dependencies, as a bit-significant mask
      */
 
@@ -456,7 +483,6 @@ public abstract class Pattern extends PseudoExpression {
         return this;
     }
 
-
     @Override
     public String toShortString() {
         return toString();
