@@ -11,6 +11,7 @@ import net.sf.saxon.expr.*;
 import net.sf.saxon.expr.instruct.DocumentInstr;
 import net.sf.saxon.expr.instruct.GlobalVariable;
 import net.sf.saxon.expr.instruct.SlotManager;
+import net.sf.saxon.expr.instruct.ValueOf;
 import net.sf.saxon.expr.parser.RoleDiagnostic;
 import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
@@ -83,6 +84,7 @@ public class SourceBinding {
         String assignableAtt = null;
         String staticAtt = null;
         String visibilityAtt = null;
+        String extraAsAtt = null;
 
         for (int a = 0; a < atts.getLength(); a++) {
             String f = atts.getQName(a);
@@ -107,8 +109,14 @@ public class SourceBinding {
                 staticAtt = Whitespace.trim(atts.getValue(a));
             } else if (f.equals("visibility") && ((permittedAttributes & VISIBILITY) != 0)) {
                 visibilityAtt = Whitespace.trim(atts.getValue(a));
-            } else if (atts.getLocalName(a).equals("assignable") && NamespaceConstant.SAXON.equals(atts.getURI(a)) && ((permittedAttributes & ASSIGNABLE) != 0)) {
-                assignableAtt = Whitespace.trim(atts.getValue(a));
+            } else if (NamespaceConstant.SAXON.equals(atts.getURI(a))) {
+                if (atts.getLocalName(a).equals("assignable") && ((permittedAttributes & ASSIGNABLE) != 0)) {
+                    assignableAtt = Whitespace.trim(atts.getValue(a));
+                } else if (atts.getLocalName(a).equals("as")) {
+                    extraAsAtt = atts.getValue(a);
+                } else {
+                    sourceElement.checkUnknownAttribute(atts.getNodeName(a));
+                }
             } else {
                 sourceElement.checkUnknownAttribute(atts.getNodeName(a));
             }
@@ -155,13 +163,7 @@ public class SourceBinding {
             }
         }
 
-        if (asAtt != null) {
-            try {
-                declaredType = sourceElement.makeSequenceType(asAtt);
-            } catch (XPathException e) {
-                sourceElement.compileErrorInAttribute(e.getMessage(), e.getErrorCodeLocalPart(), "as");
-            }
-        }
+        declaredType = combineTypeAttributes(asAtt, extraAsAtt);
 
         if (visibilityAtt != null) {
             if (hasProperty(PARAM)) {
@@ -180,6 +182,85 @@ public class SourceBinding {
         }
     }
 
+    /**
+     * This method is called to establish basic signature information for local parameters of a named template,
+     * so that this can be checked against xsl:call-template instructions. It is called while a named template
+     * is being indexed: see bug 3722.
+     * @throws XPathException
+     */
+
+    public void prepareTemplateSignatureAttributes() throws XPathException {
+
+        AttributeCollection atts = sourceElement.getAttributeList();
+
+        String asAtt = null;
+        String extraAsAtt = null;
+
+        for (int a = 0; a < atts.getLength(); a++) {
+            String f = atts.getQName(a);
+            if (f.equals("name")) {
+                if (name == null || name.equals(errorName())) {
+                    processVariableName(atts.getValue(a));
+                }
+            } else if (f.equals("as")) {
+                asAtt = atts.getValue(a);
+            } else if (f.equals("required")) {
+                String requiredAtt = Whitespace.trim(atts.getValue(a));
+                boolean required = sourceElement.processBooleanAttribute("required", requiredAtt);
+                setProperty(REQUIRED, required);
+            } else if (f.equals("tunnel")) {
+                String tunnelAtt = Whitespace.trim(atts.getValue(a));
+                boolean tunnel = sourceElement.processBooleanAttribute("tunnel", tunnelAtt);
+                setProperty(TUNNEL, tunnel);
+            } else if (NamespaceConstant.SAXON.equals(atts.getURI(a))) {
+                if (atts.getLocalName(a).equals("as")) {
+                    extraAsAtt = atts.getValue(a);
+                }
+            }
+        }
+
+        if (name == null) {
+            sourceElement.reportAbsence("name");
+            name = errorName();
+        }
+
+        declaredType = combineTypeAttributes(asAtt, extraAsAtt);
+    }
+
+    private SequenceType combineTypeAttributes(String asAtt, String extraAsAtt) throws XPathException {
+        SequenceType declaredType = null;
+        if (asAtt != null) {
+            try {
+                declaredType = sourceElement.makeSequenceType(asAtt);
+            } catch (XPathException e) {
+                sourceElement.compileErrorInAttribute(e.getMessage(), e.getErrorCodeLocalPart(), "as");
+            }
+        }
+
+        if (extraAsAtt != null) {
+            SequenceType extraResultType = null;
+            try {
+                extraResultType = sourceElement.makeExtendedSequenceType(extraAsAtt);
+            } catch (XPathException e) {
+                sourceElement.compileErrorInAttribute(e.getMessage(), e.getErrorCodeLocalPart(), "saxon:as");
+                extraResultType = SequenceType.ANY_SEQUENCE;
+            }
+            if (asAtt != null) {
+                int rel = sourceElement.getConfiguration().getTypeHierarchy().sequenceTypeRelationship(extraResultType, declaredType);
+                if (rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMED_BY) {
+                    declaredType = extraResultType;
+                } else {
+                    sourceElement.compileErrorInAttribute("When both are present, @saxon:as must be a subtype of @as", "SXER7TBA", "as");
+                }
+            } else {
+                declaredType = extraResultType;
+            }
+        }
+
+        return declaredType;
+    }
+
+
     /**
      * Get the declaration in the stylesheet
      *
@@ -449,7 +530,11 @@ public class SourceBinding {
                     b = Literal.makeEmptySequence();
                 }
                 boolean textonly = UType.TEXT.subsumes(b.getItemType().getUType());
-                DocumentInstr doc = new DocumentInstr(textonly, constantText);
+                String constant = null;  // bug 3748
+                if (textonly && b instanceof ValueOf && ((ValueOf)b).getSelect() instanceof StringLiteral) {
+                    constant = ((StringLiteral)((ValueOf) b).getSelect()).getStringValue();
+                }
+                DocumentInstr doc = new DocumentInstr(textonly, constant);
                 doc.setContentExpression(b);
                 doc.setRetainedStaticContext(sourceElement.makeRetainedStaticContext());
                 select = doc;
