@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
-// Copyright (c) 2015 Saxonica Limited.
+// Copyright (c) 2017 Saxonica Limited.
 // This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
 // If a copy of the MPL was not distributed with this file, You can obtain one at http://mozilla.org/MPL/2.0/.
 // This Source Code Form is "Incompatible With Secondary Licenses", as defined by the Mozilla Public License, v. 2.0.
@@ -13,26 +13,35 @@ import net.sf.saxon.expr.Component;
 import net.sf.saxon.expr.ComponentBinding;
 import net.sf.saxon.expr.PackageData;
 import net.sf.saxon.expr.instruct.*;
-import net.sf.saxon.functions.*;
-import net.sf.saxon.om.NoElementsSpaceStrippingRule;
+import net.sf.saxon.expr.parser.OptimizerOptions;
+import net.sf.saxon.functions.ExecutableFunctionLibrary;
+import net.sf.saxon.functions.FunctionLibrary;
+import net.sf.saxon.functions.FunctionLibraryList;
+import net.sf.saxon.functions.registry.ConstructorFunctionLibrary;
 import net.sf.saxon.om.SpaceStrippingRule;
-import net.sf.saxon.om.StandardNames;
 import net.sf.saxon.om.StructuredQName;
+import net.sf.saxon.pattern.NodeTest;
 import net.sf.saxon.query.XQueryFunctionLibrary;
 import net.sf.saxon.serialize.CharacterMap;
 import net.sf.saxon.serialize.CharacterMapIndex;
 import net.sf.saxon.trace.ExpressionPresenter;
 import net.sf.saxon.trans.*;
+import net.sf.saxon.trans.rules.RuleManager;
 import net.sf.saxon.tree.util.FastStringBuffer;
+import net.sf.saxon.type.TypeHierarchy;
 
 import java.util.*;
 
+import static net.sf.saxon.trans.Visibility.PRIVATE;
+
 /**
  * A (compiled) stylesheet package. This may be created either by compiling a source XSLT package,
  * or by loading a saved package from disk. It therefore has no references to source XSLT documents.
  */
 public class StylesheetPackage extends PackageData {
 
+    private final static boolean TRACING = false;
+
     /**
      * A class that simply encapsulates a callback action of any kind
      */
@@ -60,17 +69,19 @@ public class StylesheetPackage extends PackageData {
     private boolean stripsTypeAnnotations = false;
     protected Properties defaultOutputProperties;
     private StructuredQName defaultMode;
+    private boolean declaredModes;
     protected Map<StructuredQName, Properties> namedOutputProperties = new HashMap<StructuredQName, Properties>(4);
 
     // table of imported schemas. The members of this set are strings holding the target namespace.
     protected Set<String> schemaIndex = new HashSet<String>(10);
 
 
-    FunctionLibraryList functionLibrary;
-    XQueryFunctionLibrary queryFunctions;
-    ExecutableFunctionLibrary overriding;
-    ExecutableFunctionLibrary underriding;
+    private FunctionLibraryList functionLibrary;
+    private XQueryFunctionLibrary queryFunctions;
+    private ExecutableFunctionLibrary overriding;
+    private ExecutableFunctionLibrary underriding;
     private int maxFunctionArity = -1;
+    private boolean retainUnusedFunctions = false;
 
 
     // Table of components declared in this package or imported from used packages. Key is the symbolic identifier of the
@@ -80,7 +91,7 @@ public class StylesheetPackage extends PackageData {
     private HashMap<SymbolicName, Component> componentIndex =
         new HashMap<SymbolicName, Component>(20);
 
-    protected HashMap<SymbolicName, Component> hiddenComponents = new HashMap<SymbolicName, Component>();
+    protected List<Component> hiddenComponents = new ArrayList<Component>();
 
     protected HashMap<SymbolicName, Component> overriddenComponents = new HashMap<SymbolicName, Component>();
 
@@ -120,7 +131,7 @@ public class StylesheetPackage extends PackageData {
     }
 
     /**
-     * Add a package to the list of packages used by this one
+     * Add a package to the list of packages directly used by this one
      * @param pack the used package
      */
 
@@ -208,10 +219,11 @@ public class StylesheetPackage extends PackageData {
         this.ruleManager = ruleManager;
     }
 
+
     /**
      * Get the default mode for the package
-     * @return the default mode
-     * TODO: the default mode can now be defined on any element
+     * @return the default mode for the package as a whole (this may change for individual elements/subtrees within
+     * the package)
      */
 
     public StructuredQName getDefaultMode() {
@@ -227,8 +239,29 @@ public class StylesheetPackage extends PackageData {
         this.defaultMode = defaultMode;
     }
 
+    /**
+     * Say whether it is required that modes be explicitly declared
+     *
+     * @param declared true if modes referenced within this package must be explicitly declared
+     */
+
+    public void setDeclaredModes(boolean declared) {
+        declaredModes = declared;
+    }
+
+    /**
+     * Ask whether it is required that modes be explicitly declared
+     *
+     * @return true if modes referenced within this package must be explicitly declared
+     */
+
+    public boolean isDeclaredModes() {
+        return declaredModes;
+    }
+
     /**
      * Get the whitespace stripping rules for this package
+     *
      * @return the whitespace stripping rules (based on xsl:strip-space and xsl:preserve-space)
      */
 
@@ -359,6 +392,18 @@ public class StylesheetPackage extends PackageData {
      */
 
     public void setContextItemRequirements(GlobalContextRequirement requirement) throws XPathException {
+        if (containsGlobalContextItemDeclaration) {
+            // the new requirements must be consistent with the existing requirements
+            if ((!requirement.isAbsentFocus() && globalContextRequirement.isAbsentFocus()) ||
+                    (requirement.isMayBeOmitted() && !globalContextRequirement.isMayBeOmitted())) {
+                throw new XPathException(
+                        "The package contains two xsl:global-context-item declarations with conflicting @use attributes", "XTSE3087");
+            }
+            TypeHierarchy th = getConfiguration().getTypeHierarchy();
+            if (th.relationship(requirement.getRequiredItemType(), globalContextRequirement.getRequiredItemType()) != TypeHierarchy.SAME_TYPE) {
+                throw new XPathException("The package contains two xsl:global-context-item declarations with conflicting item types", "XTSE3087");
+            }
+        }
         containsGlobalContextItemDeclaration = true;
         globalContextRequirement = requirement;
     }
@@ -383,6 +428,16 @@ public class StylesheetPackage extends PackageData {
         this.stripsWhitespace = strips;
     }
 
+    /**
+     * Ask whether there is an xsl:strip-space declaration in the stylesheet package
+     *
+     * @return true if an xsl:strip-space declaration has been found
+     */
+
+    public boolean isStripsWhitespace() {
+        return this.stripsWhitespace;
+    }
+
     /**
      * Register a callback action to be performed during the completion phase of building the package
      *
@@ -416,7 +471,7 @@ public class StylesheetPackage extends PackageData {
         for (Component c : componentIndex.values()) {
             registerGlobalVariable(c, slotManager);
         }
-        for (Component c : hiddenComponents.values()) {
+        for (Component c : hiddenComponents) {
             registerGlobalVariable(c, slotManager);
         }
         setGlobalSlotManager(slotManager);
@@ -432,12 +487,14 @@ public class StylesheetPackage extends PackageData {
      */
 
     private void registerGlobalVariable(Component c, SlotManager slotManager) {
-        if (c.getCode() instanceof GlobalVariable) {
-            GlobalVariable var = (GlobalVariable) c.getCode();
+        if (c.getActor() instanceof GlobalVariable) {
+            GlobalVariable var = (GlobalVariable) c.getActor();
             int slot = slotManager.allocateSlotNumber(var.getVariableQName());
             var.setPackageData(this);
             var.setBinderySlotNumber(slot);
-            addGlobalVariable(var);
+            if (c.getVisibility() != Visibility.HIDDEN) {
+                addGlobalVariable(var);
+            }
         }
     }
 
@@ -447,10 +504,10 @@ public class StylesheetPackage extends PackageData {
      */
 
     public void addComponent(Component component) {
-        SymbolicName name = component.getCode().getSymbolicName();
+        SymbolicName name = component.getActor().getSymbolicName();
         componentIndex.put(name, component);
-        if (component.getVisibility() == Visibility.ABSTRACT) {
-            abstractComponents.put(component.getCode().getSymbolicName(), component);
+        if (component.getVisibility() == Visibility.ABSTRACT && component.getContainingPackage() == this) {
+            abstractComponents.put(component.getActor().getSymbolicName(), component);
         }
     }
 
@@ -459,9 +516,9 @@ public class StylesheetPackage extends PackageData {
         super.addGlobalVariable(variable);
         SymbolicName name = variable.getSymbolicName();
         if (componentIndex.get(name) == null) {
-            Component<GlobalVariable> comp = variable.getDeclaringComponent();
+            Component comp = variable.getDeclaringComponent();
             if (comp == null) {
-                comp = variable.makeDeclaringComponent(Visibility.PRIVATE, this);
+                comp = variable.makeDeclaringComponent(PRIVATE, this);
             }
             addComponent(comp);
         }
@@ -475,9 +532,9 @@ public class StylesheetPackage extends PackageData {
     public int getMaxFunctionArity() {
         if (maxFunctionArity == -1) {
             for (Component c : componentIndex.values()) {
-                if (c.getCode() instanceof UserFunction) {
-                    if (((UserFunction) c.getCode()).getArity() > maxFunctionArity) {
-                        maxFunctionArity = ((UserFunction) c.getCode()).getArity();
+                if (c.getActor() instanceof UserFunction) {
+                    if (((UserFunction) c.getActor()).getArity() > maxFunctionArity) {
+                        maxFunctionArity = ((UserFunction) c.getActor()).getArity();
                     }
                 }
             }
@@ -496,12 +553,12 @@ public class StylesheetPackage extends PackageData {
         return componentIndex.get(name);
     }
 
-    public Component getHiddenComponent(SymbolicName name) {
-        return hiddenComponents.get(name);
-    }
+//    public Component getHiddenComponent(SymbolicName name) {
+//        return hiddenComponents.get(name);
+//    }
 
     public void addHiddenComponent(Component component) {
-        hiddenComponents.put(component.getCode().getSymbolicName(), component);
+        hiddenComponents.add(component);
     }
 
     /**
@@ -517,7 +574,7 @@ public class StylesheetPackage extends PackageData {
     }
 
     public void addOverriddenComponent(Component comp) {
-        overriddenComponents.put(comp.getCode().getSymbolicName(), comp);
+        overriddenComponents.put(comp.getActor().getSymbolicName(), comp);
     }
 
     /**
@@ -530,9 +587,12 @@ public class StylesheetPackage extends PackageData {
      */
 
     public void addComponentsFromUsedPackage(StylesheetPackage usedPackage,
-                                             List<ComponentAcceptor> acceptors,
+                                             List<XSLAccept> acceptors,
                                              final Set<SymbolicName> overrides) throws XPathException {
         usedPackages.add(usedPackage);
+
+        trace("=== Adding components from " + usedPackage.getPackageName() + " to " + getPackageName() + " ===");
+
         // Create copies of the components in the used package, with suitably adjusted visibility
 
         // Create a mapping from components in the used package to their corresponding components
@@ -542,14 +602,44 @@ public class StylesheetPackage extends PackageData {
 
         for (Map.Entry<SymbolicName, Component> namedComponentEntry : usedPackage.componentIndex.entrySet()) {
             SymbolicName name = namedComponentEntry.getKey();
-            Component<? extends ComponentCode> oldC = namedComponentEntry.getValue();
+            Component oldC = namedComponentEntry.getValue();
 
-            // Spec section 3.6.2.4: The visibility of C(P) will be the same as the visibility of C(Q),
-            // except that where the visibility of C(Q) is private, the visibility of C(P) will be hidden.
             Visibility oldV = oldC.getVisibility();
-            Visibility newV = oldV == Visibility.PRIVATE ? Visibility.HIDDEN : oldV;
 
-            final Component<ComponentCode> newC = new Component<ComponentCode>(oldC.getCode(), newV, this, oldC.getDeclaringPackage());
+            Visibility newV = null;
+
+            if (overrides.contains(name) && !(oldC.getActor() instanceof Mode)) {
+                newV = Visibility.HIDDEN;
+            } else {
+                Visibility acceptedVisibility = explicitAcceptedVisibility(name, acceptors);
+                if (acceptedVisibility != null) {
+                    if (!XSLAccept.isCompatible(oldV, acceptedVisibility)) {
+                        throw new XPathException("Cannot accept a " + oldV.toString() +
+                            " component with visibility " + acceptedVisibility.toString() +
+                        " (" + name + ")", "XTSE3040");
+                    }
+                    newV = acceptedVisibility;
+                } else {
+                    acceptedVisibility = wildcardAcceptedVisibility(name, acceptors);
+                    if (acceptedVisibility != null) {
+                        if (XSLAccept.isCompatible(oldV, acceptedVisibility)) {
+                            newV = acceptedVisibility;
+                        }
+                    }
+                }
+
+                if (newV == null) {
+                    if (oldV == Visibility.PUBLIC || oldV == Visibility.FINAL) {
+                        newV = Visibility.PRIVATE;
+                    } else {
+                        newV = Visibility.HIDDEN;
+                    }
+                }
+            }
+
+            trace(oldC.getActor().getSymbolicName() + " (" + oldV + ") becomes " + newV);
+
+            final Component newC = Component.makeComponent(oldC.getActor(), newV, this, oldC.getDeclaringPackage());
             correspondence.put(oldC, newC);
             newC.setBaseComponent(oldC);
 
@@ -559,42 +649,101 @@ public class StylesheetPackage extends PackageData {
                 if (newV != Visibility.ABSTRACT) {
                     abstractComponents.remove(name);
                 }
-            } else {
-                if (newV != Visibility.HIDDEN) {
-                    for (ComponentAcceptor acceptor : acceptors) {
-                        acceptor.acceptComponent(newC);
-                    }
-                }
-                if (oldV == Visibility.ABSTRACT) {
-                    for (ComponentAcceptor acceptor : acceptors) {
-                        acceptor.acceptComponent(newC);
-                    }
-                    if (newC.getVisibility() == Visibility.ABSTRACT) {
-                        abstractComponents.put(name, newC);
-                    }
-                }
             }
-
             if (newC.getVisibility() == Visibility.HIDDEN) {
-                hiddenComponents.put(namedComponentEntry.getKey(), newC);
+                hiddenComponents.add(newC);
             } else if (componentIndex.get(name) != null) {
-                if (!(oldC.getCode() instanceof Mode)) {
-                    throw new XPathException("Duplicate " + namedComponentEntry.getKey(), "XTSE3050", oldC.getCode());
+                if (!(oldC.getActor() instanceof Mode)) {
+                    throw new XPathException("Duplicate " + namedComponentEntry.getKey(), "XTSE3050", oldC.getActor());
                 }
             } else {
                 componentIndex.put(name, newC);
-                if (oldC.getCode() instanceof Mode && (oldV == Visibility.PUBLIC || oldV == Visibility.FINAL) ) {
+                if (oldC.getActor() instanceof Mode && (oldV == Visibility.PUBLIC || oldV == Visibility.FINAL) ) {
                     Mode existing = getRuleManager().obtainMode(name.getComponentName(), false);
                     if (existing != null) {
-                        throw new XPathException("Duplicate " + namedComponentEntry.getKey(), "XTSE3050", oldC.getCode());
+                        throw new XPathException("Duplicate " + namedComponentEntry.getKey(), "XTSE3050", oldC.getActor());
                     } else {
                         //getRuleManager().registerMode((Mode)oldC.getCode());
                     }
                 }
             }
 
+            if (newC.getActor() instanceof Mode && overrides.contains(name)) {
+                addCompletionAction(new Action() {
+                    @Override
+                    public void doAction() throws XPathException {
+                        trace("Doing mode completion for " + newC.getActor().getSymbolicName());
+                        List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
+                        List<ComponentBinding> newBindings = newC.getComponentBindings();
+                        for (int i=0; i<oldBindings.size(); i++) {
+                            SymbolicName name = oldBindings.get(i).getSymbolicName();
+                            Component target;
+                            if (overrides.contains(name)) {
+                                // if there is an override in this package, we bind to it
+                                target = getComponent(name);
+                                if (target == null) {
+                                    throw new AssertionError("We know there's an override for " + name + ", but we can't find it");
+                                }
+                            } else {
+                                // otherwise we bind to the component in this package that corresponds to the component in the used package
+                                target = correspondence.get(oldBindings.get(i).getTarget());
+                                if (target == null) {
+                                    throw new AssertionError("Saxon can't find the new component corresponding to " + name);
+                                }
+                            }
+                            ComponentBinding newBinding = new ComponentBinding(name, target);
+                            newBindings.set(i, newBinding);
+                        }
+
+                    }
+                });
+            } else {
+
+                addCompletionAction(new Action() {
+                    @Override
+                    public void doAction() throws XPathException {
+                        trace("Doing normal completion for " + newC.getActor().getSymbolicName());
+                        List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
+                        List<ComponentBinding> newBindings = new ArrayList<ComponentBinding>(oldBindings.size());
+                        for (ComponentBinding oldBinding : oldBindings) {
+                            SymbolicName name = oldBinding.getSymbolicName();
+                            Component target;
+                            if (overrides.contains(name)) {
+                                // if there is an override in this package, we bind to it
+                                target = getComponent(name);
+                                if (target == null) {
+                                    throw new AssertionError("We know there's an override for " + name + ", but we can't find it");
+                                }
+                            } else {
+                                // otherwise we bind to the component in this package that corresponds to the component in the used package
+                                target = correspondence.get(oldBinding.getTarget());
+                                if (target == null) {
+                                    throw new AssertionError("Saxon can't find the new component corresponding to " + name);
+                                }
+                            }
+                            ComponentBinding newBinding = new ComponentBinding(name, target);
+                            newBindings.add(newBinding);
+                        }
+                        newC.setComponentBindings(newBindings);
+
+                    }
+                });
+            }
+
+        }
+
+        for (Component oldC : usedPackage.hiddenComponents) {
+
+            trace(oldC.getActor().getSymbolicName() + " (HIDDEN, declared in " + oldC.getDeclaringPackage().getPackageName() + ") becomes HIDDEN");
+
+            final Component newC = Component.makeComponent(oldC.getActor(), Visibility.HIDDEN, this, oldC.getDeclaringPackage());
+            correspondence.put(oldC, newC);
+            newC.setBaseComponent(oldC);
+
+            hiddenComponents.add(newC);
 
             addCompletionAction(new Action() {
+                // TODO: common code with above
                 @Override
                 public void doAction() throws XPathException {
                     List<ComponentBinding> oldBindings = newC.getBaseComponent().getComponentBindings();
@@ -605,13 +754,13 @@ public class StylesheetPackage extends PackageData {
                         if (overrides.contains(name)) {
                             // if there is an override in this package, we bind to it
                             target = getComponent(name);
-                            if (target==null) {
+                            if (target == null) {
                                 throw new AssertionError("We know there's an override for " + name + ", but we can't find it");
                             }
                         } else {
                             // otherwise we bind to the component in this package that corresponds to the component in the used package
                             target = correspondence.get(oldBinding.getTarget());
-                            if (target==null) {
+                            if (target == null) {
                                 throw new AssertionError("Saxon can't find the new component corresponding to " + name);
                             }
                         }
@@ -623,11 +772,53 @@ public class StylesheetPackage extends PackageData {
             });
 
         }
+
         if (usedPackage.isCreatesSecondaryResultDocuments()) {
             setCreatesSecondaryResultDocuments(true);
         }
     }
 
+    private void trace(String message) {
+        if (TRACING) {
+            System.err.println(message);
+        }
+    }
+
+    private Visibility explicitAcceptedVisibility(SymbolicName name, List<XSLAccept> acceptors) throws XPathException {
+        for (XSLAccept acceptor : acceptors) {
+            for (ComponentTest test : acceptor.getExplicitComponentTests()) {
+                if (test.matches(name)) {
+                    return acceptor.getVisibility();
+                }
+            }
+        }
+        return null;
+    }
+
+    private Visibility wildcardAcceptedVisibility(SymbolicName name, List<XSLAccept> acceptors) throws XPathException {
+        // Note: last one wins
+        Visibility vis = null;
+        for (XSLAccept acceptor : acceptors) {
+            for (ComponentTest test : acceptor.getWildcardComponentTests()) {
+                if (((NodeTest) test.getQNameTest()).getDefaultPriority() == -0.25 && test.matches(name)) {
+                    vis = acceptor.getVisibility();
+                }
+            }
+        }
+        if (vis != null) {
+            return vis;
+        }
+        for (XSLAccept acceptor : acceptors) {
+            for (ComponentTest test : acceptor.getWildcardComponentTests()) {
+                if (test.matches(name)) {
+                    vis = acceptor.getVisibility();
+                }
+            }
+        }
+        return vis;
+    }
+
+
     /**
      * Create the function library containing stylesheet functions declared in this package
      *
@@ -637,21 +828,12 @@ public class StylesheetPackage extends PackageData {
     public FunctionLibraryList createFunctionLibrary() {
 
         FunctionLibraryList functionLibrary = new FunctionLibraryList();
-        int functionSet = StandardFunction.CORE | StandardFunction.XSLT;
-        if (getVersion() >= 30) {
-            functionSet |= StandardFunction.XSLT30 | StandardFunction.XPATH30 | StandardFunction.XPATH31;
-        }
-        functionLibrary.addFunctionLibrary(
-            SystemFunctionLibrary.getSystemFunctionLibrary(functionSet, config));
-        functionLibrary.addFunctionLibrary(
-            new StylesheetFunctionLibrary(this, true));
-        functionLibrary.addFunctionLibrary(
-            config.getVendorFunctionLibrary());
-        functionLibrary.addFunctionLibrary(
-            new ConstructorFunctionLibrary(config));
-        if ("JS".equals(getTargetEdition())) {
+        functionLibrary.addFunctionLibrary(config.getXSLT30FunctionSet());
+        functionLibrary.addFunctionLibrary(new StylesheetFunctionLibrary(this, true));
+        functionLibrary.addFunctionLibrary(config.getBuiltInExtensionLibraryList());
+        functionLibrary.addFunctionLibrary(new ConstructorFunctionLibrary(config));
+        if ("JS".equals(getTargetEdition()) || "JS2".equals(getTargetEdition())) {
             addIxslFunctionLibrary(functionLibrary);
-
         }
 
         queryFunctions = new XQueryFunctionLibrary(config);
@@ -710,24 +892,51 @@ public class StylesheetPackage extends PackageData {
      * @return the requested function, or null if none can be found
      */
 
-    protected UserFunction getFunction(SymbolicName name) {
+    protected UserFunction getFunction(SymbolicName.F name) {
         if (name.getArity() == -1) {
             // supports the single-argument function-available() function
             int maximumArity = 20;
             for (int a = 0; a < maximumArity; a++) {
-                SymbolicName sn = new SymbolicName(StandardNames.XSL_FUNCTION, name.getComponentName(), a);
+                SymbolicName.F sn = new SymbolicName.F(name.getComponentName(), a);
                 UserFunction uf = getFunction(sn);
                 if (uf != null) {
+                    uf.incrementReferenceCount();
                     return uf;
                 }
             }
             return null;
         } else {
             Component component = getComponentIndex().get(name);
-            return component == null ? null : (UserFunction) component.getCode();
+            if (component != null) {
+                UserFunction uf = (UserFunction) component.getActor();
+                uf.incrementReferenceCount();
+                return uf;
+            } else {
+                return null;
+            }
         }
     }
 
+    /**
+     * Ask whether private functions for which there are no static references need to be
+     * retained
+     * @return true if such functions need to be retained, typically because the stylesheet
+     * package uses xsl:evaluate or fn:function-lookup
+     */
+
+    public boolean isRetainUnusedFunctions() {
+        return retainUnusedFunctions;
+    }
+
+    /**
+     * Say that private functions need to be retained, typically because the stylesheet
+     * package uses xsl:evaluate or fn:function-lookup
+     */
+
+    public void setRetainUnusedFunctions() {
+        this.retainUnusedFunctions = true;
+    }
+
     /**
      * Copy information from this package to the PreparedStylesheet.
      *
@@ -747,7 +956,7 @@ public class StylesheetPackage extends PackageData {
         if (isSchemaAware() || !schemaIndex.isEmpty()) {
             pss.setSchemaAware(true);
         }
-        pss.setHostLanguage(Configuration.XSLT, xsltVersion >= 30);
+        pss.setHostLanguage(Configuration.XSLT);
 
         //pss.setStylesheetFunctionLibrary(stylesheetFunctionLibrary);
 
@@ -769,14 +978,6 @@ public class StylesheetPackage extends PackageData {
 
         pss.setKeyManager(keyManager);
 
-        //pss.setAccumulatorManager(accumulatorManager);
-
-        pss.setStripsWhitespace(stripsWhitespace);
-
-        pss.setStripsInputTypeAnnotations(stripsTypeAnnotations);
-
-        pss.setStripperRules(stripperRules == null ? NoElementsSpaceStrippingRule.getInstance() : stripperRules);
-
         if (!pss.createsSecondaryResult()) {
             pss.setCreatesSecondaryResult(createsSecondaryResultDocuments);
         }
@@ -798,15 +999,17 @@ public class StylesheetPackage extends PackageData {
         ruleManager.checkConsistency();
         ruleManager.computeRankings();
         ruleManager.invertStreamableTemplates();
-        ruleManager.optimizeRules();
+        if (config.obtainOptimizer().isOptionSet(OptimizerOptions.RULE_SET)) {
+            ruleManager.optimizeRules();
+        }
         pss.setRuleManager(ruleManager);
 
 
         // Add named templates to the prepared stylesheet
 
         for (Component comp : componentIndex.values()) {
-            if (comp.getCode() instanceof NamedTemplate) {
-                NamedTemplate t = (NamedTemplate) comp.getCode();
+            if (comp.getActor() instanceof NamedTemplate) {
+                NamedTemplate t = (NamedTemplate) comp.getActor();
                 pss.putNamedTemplate(t.getTemplateName(), t);
             }
         }
@@ -818,8 +1021,8 @@ public class StylesheetPackage extends PackageData {
         // Register stylesheet parameters
 
         for (Component comp : componentIndex.values()) {
-            if (comp.getCode() instanceof GlobalParam) {
-                GlobalParam gv = (GlobalParam) comp.getCode();
+            if (comp.getActor() instanceof GlobalParam) {
+                GlobalParam gv = (GlobalParam) comp.getActor();
                 pss.registerGlobalParameter(gv);
             }
         }
@@ -848,7 +1051,7 @@ public class StylesheetPackage extends PackageData {
 
     public void checkForAbstractComponents() throws XPathException {
         for (Map.Entry<SymbolicName, Component> entry : componentIndex.entrySet()) {
-            if (entry.getValue().getVisibility() == Visibility.ABSTRACT) {
+            if (entry.getValue().getVisibility() == Visibility.ABSTRACT && entry.getValue().getContainingPackage() == this) {
                 abstractComponents.put(entry.getKey(), entry.getValue());
             }
         }
@@ -871,6 +1074,14 @@ public class StylesheetPackage extends PackageData {
         }
     }
 
+    public boolean isJustInTimeCompilation() {
+        return false; // not available in Saxon-HE
+    }
+
+    public void setJustInTimeCompilation(boolean justInTimeCompilation) {
+        // no action for Saxon-HE
+    }
+
 
 }
 
