@@ -13,6 +13,7 @@ import net.sf.saxon.expr.instruct.*;
 import net.sf.saxon.expr.parser.*;
 import net.sf.saxon.lib.FeatureKeys;
 import net.sf.saxon.lib.Logger;
+import net.sf.saxon.lib.NamespaceConstant;
 import net.sf.saxon.om.*;
 import net.sf.saxon.pattern.Pattern;
 import net.sf.saxon.trace.LocationKind;
@@ -24,6 +25,7 @@ import net.sf.saxon.tree.linked.NodeImpl;
 import net.sf.saxon.type.AnyItemType;
 import net.sf.saxon.type.ErrorType;
 import net.sf.saxon.type.ItemType;
+import net.sf.saxon.type.TypeHierarchy;
 import net.sf.saxon.value.BigDecimalValue;
 import net.sf.saxon.value.SequenceType;
 import net.sf.saxon.value.Whitespace;
@@ -62,6 +64,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
     //private boolean maySupplyContextItem = true;
     private boolean absentFocus = false;
     //private Expression body;
+    private boolean jitCompilationDone = false;
 
     /**
      * Get the corresponding NamedTemplate object that results from the compilation of this
@@ -278,6 +281,8 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
 
     public void prepareAttributes() throws XPathException {
 
+        String extraAsAtt = null;
+
         AttributeCollection atts = getAttributeList();
 
         for (int a = 0; a < atts.getLength(); a++) {
@@ -294,6 +299,10 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 asAtt = atts.getValue(a);
             } else if (f.equals("visibility")) {
                 visibilityAtt = Whitespace.trim(atts.getValue(a));
+            } else if (atts.getURI(a).equals(NamespaceConstant.SAXON)) {
+                if (atts.getLocalName(a).equals("as")) {
+                    extraAsAtt = atts.getValue(a);
+                }
             } else {
                 checkUnknownAttribute(atts.getNodeName(a));
             }
@@ -379,6 +388,26 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
             }
         }
 
+        if (extraAsAtt != null) {
+            SequenceType extraResultType = null;
+            try {
+                extraResultType = makeExtendedSequenceType(extraAsAtt);
+            } catch (XPathException e) {
+                compileErrorInAttribute(e.getMessage(), e.getErrorCodeLocalPart(), "saxon:as");
+                extraResultType = requiredType; // error recovery
+            }
+            if (asAtt != null) {
+                int rel = getConfiguration().getTypeHierarchy().sequenceTypeRelationship(extraResultType, requiredType);
+                if (rel == TypeHierarchy.SAME_TYPE || rel == TypeHierarchy.SUBSUMED_BY) {
+                    requiredType = extraResultType;
+                } else {
+                    compileErrorInAttribute("When both are present, @saxon:as must be a subtype of @as", "SXER7TBA", "saxon:as");
+                }
+            } else {
+                requiredType = extraResultType;
+            }
+        }
+
         if (visibilityAtt != null) {
             check30attribute("visibility");
             visibility = interpretVisibilityValue(visibilityAtt, "");
@@ -782,6 +811,38 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
         templateRule.setContextItemRequirements(requiredContextItemType, absentFocus);
     }
 
+    public synchronized void jitCompile(Compilation compilation, ComponentDeclaration decl) throws XPathException {
+        if (!jitCompilationDone) {
+            compilation.setPreScan(false);
+            processAllAttributes();
+
+            if (compilation.getErrorCount() > 0) {
+                XPathException e = new XPathException("Errors were reported during stylesheet compilation");
+                e.setHasBeenReported(true); // only intended as an exception message, not something to report to ErrorListener
+                throw e;
+            }
+
+            validateSubtree(decl, false);
+
+            if (compilation.getErrorCount() > 0) {
+                XPathException e = new XPathException("Errors were reported during stylesheet compilation");
+                e.setHasBeenReported(true); // only intended as an exception message, not something to report to ErrorListener
+                throw e;
+            }
+
+            compileDeclaration(compilation, decl);
+            // xslTemplate.optimize(decl);
+            //allocateBindingSlots();
+            if (compilation.getErrorCount() > 0) {
+                XPathException e = new XPathException("Errors were reported during stylesheet compilation");
+                e.setHasBeenReported(true); // only intended as an exception message, not something to report to ErrorListener
+                throw e;
+            }
+            jitCompilationDone = true;
+        }
+
+    }
+
     /**
      * Registers the template rule with each Mode that it belongs to.
      *
@@ -811,16 +872,17 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                         return;
                     }
                 }
-                Pattern match2 = match;
+                Pattern match1 = match.copy(new RebindingMap());
                 String typed = mode.getActivePart().getPropertyValue("typed");
                 if ("strict".equals(typed) || "lax".equals(typed)) {
+                    Pattern match2;
                     try {
-                        match2 = match.convertToTypedPattern(typed);
+                        match2 = match1.convertToTypedPattern(typed);
                     } catch (XPathException e) {
                         e.maybeSetLocation(this);
                         throw e;
                     }
-                    if (match2 != match) {
+                    if (match2 != match1) {
                         ContextItemStaticInfo info = getConfiguration().makeContextItemStaticInfo(AnyItemType.getInstance(), false);
                         ExpressionTool.copyLocationInfo(match, match2);
                         //match2.setPackageData(match.getPackageData());
@@ -828,6 +890,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                         //match2.setSystemId(match.getSystemId());
                         //match2.setLineNumber(match.getLineNumber());
                         match2 = match2.typeCheck(visitor, info);
+                        match1 = match2;
                     }
                     if (modeNames.length == 1) {
                         // If this is the only mode for the template, then we can use this enhanced match pattern
@@ -844,11 +907,11 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                 }
 
                 double prio = prioritySpecified ? priority : Double.NaN;
-                mgr.setTemplateRule(match2, rule, mode, module, prio);
+                mgr.setTemplateRule(match1, rule, mode, module, prio);
 
                 if (mode.isDeclaredStreamable()) {
                     rule.setDeclaredStreamable(true);
-                    if (!match2.isMotionless()) {
+                    if (!match1.isMotionless()) {
                         boolean fallback = getConfiguration().getBooleanProperty(FeatureKeys.STREAMING_FALLBACK);
                         String message = "Template rule is declared streamable but the match pattern is not motionless";
                         if (fallback) {
@@ -865,7 +928,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
 
                 if (mode.getModeName().equals(Mode.OMNI_MODE)) {
                     compiledTemplateRules.put(Mode.UNNAMED_MODE_NAME, rule);
-                    mgr.setTemplateRule(match2, rule, mgr.getUnnamedMode(), module, prio);
+                    mgr.setTemplateRule(match1, rule, mgr.getUnnamedMode(), module, prio);
                     for (Mode m : mgr.getAllNamedModes()) {
                         if (m instanceof SimpleMode) {
                             TemplateRule ruleCopy = rule.copy();
@@ -873,7 +936,7 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                                 ruleCopy.setDeclaredStreamable(true);
                             }
                             compiledTemplateRules.put(m.getModeName(), ruleCopy);
-                            mgr.setTemplateRule(match2.copy(new RebindingMap()), ruleCopy, m, module, prio);
+                            mgr.setTemplateRule(match1.copy(new RebindingMap()), ruleCopy, m, module, prio);
                         }
                     }
                 }
@@ -899,6 +962,11 @@ public final class XSLTemplate extends StyleElement implements StylesheetCompone
                         nextFree = 1;
                     }
                     int slots = match.allocateSlots(getSlotManager(), nextFree);
+                    // if the pattern calls user-defined functions, allocate at least one slot,
+                    // to force a new context to be created for evaluating patterns (bug 3706)
+                    if (slots == 0 && ((match.getDependencies() & StaticProperty.DEPENDS_ON_USER_FUNCTIONS) != 0)) {
+                        slots = 1;
+                    }
                     if (slots > 0) {
                         RuleManager mgr = getCompilation().getPrincipalStylesheetModule().getRuleManager();
                         boolean appliesToAll = false;
