@@ -298,6 +298,9 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
                 XsltPackage xpack = compiler.compilePackage(styleSource2);
                 xpack = exportImportPackage(testName, testSetName, outcome, compiler, xpack, collector);
                 compiler.importPackage(xpack);
+                // Following needed for dynamic loading of packages using fn:transform()
+                env.processor.getUnderlyingConfiguration().getDefaultXsltCompilerInfo().getPackageLibrary().addPackage(xpack.getUnderlyingPreparedPackage());
+
             }
             sheet = exportImport(testName, testSetName, outcome, compiler, sheet, collector, styleSource);
 
@@ -814,6 +817,7 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
             } else {
                 outcome.setErrorsReported(collector.getErrorCodes());
             }
+            return false;
         } catch (Exception err) {
             err.printStackTrace();
             noteFailure(testSetName, testName);
@@ -1056,8 +1060,8 @@ public class Xslt30TestSuiteDriverHE extends TestDriver {
         alwaysOn.add("unparsed_text_encoding");
         alwaysOn.add("recognize_id_as_uri_fragment");
         alwaysOn.add("feature/XPath_3.1");
+        alwaysOn.add("feature/backwards_compatibility");
 
-        needsPE.add("feature/backwards_compatibility");
         needsPE.add("feature/Saxon-PE");
         needsPE.add("feature/dynamic_evaluation");
 
